
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientFileSysBase.h
**
** DESCRIPTION: This is header file to define the class:
**              CClientFileSysBase
**
**              This class mainly manage the file system used by the load
**              generator.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
**  CLASS LIST:
**      CClientFileSysBase
**
**
****************************************************************************/

#ifndef _CLIENTFILESYSBASE_H_
#define _CLIENTFILESYSBASE_H_

#include "Common.h"

class CSocketClient;

#define FILESIZEDIST_MAXITEMNUM         16

struct SFileSizeDist
{
    int num;                            /* number of item */
    struct
    {
        int pcnt;                       /* percentile */
        int size;                       /* file size in KBytes */
    } item[FILESIZEDIST_MAXITEMNUM];
};

#define SUBDIR_MAXMNUM                  997

class CClientFileSysBase
{
protected:
    /* used to refresh file system */
    int m_job_id;                       /* identity of this job */
    int m_client_id;                    /* identity of client in this job executing process */
    char m_HostName[MAX_PATHLEN];       /* host name of resource manager server or primary client */
    char m_HostIp[MAX_IPADDRLEN];       /* ip address of resource manager server or primary client */
    BOOL m_bUpdate;                     /* whether to update refresh progress */

    BOOL m_noac;                        /* nfs mount option */
    BOOL m_mounted;                     /* file system is mounted correctly */

    char m_ServerName[MAX_NAMELEN];     /* file server name or ip address */
    char m_ServerNetIF[MAX_NAMELEN];    /* file server network interface name or ip address */
    char m_ServerPath[MAX_PATHLEN];     /* file system path on file server */
    char m_LocalPath[MAX_PATHLEN];      /* client path which file system is mounted on */

    int m_TotalSize;                    /* file system size (max is 2048GB) in KBytes */
    int m_TotalFNodeNum;                /* total number of file nodes (inodes) */
    SFileSizeDist m_IODist;             /* file size distribution for IO operations */
    int m_SubDirNum;                    /* files is put into different subdirectory, this is dir number */
    int m_TotalIONum;                   /* total file number to do IO operations */
    int m_TotalFileNum;                 /* total file number to do NonIO operations */
    int m_TotalDirNum;                  /* total directory number to do directory operations */
    int m_TotalSymlinkNum;              /* total symlink number to do directory operations */

    int *m_IONum;                       /* the number of IO file in each sub directory */
    int **m_IOSize;                     /* size of each IO file in each sub directory */
    int *m_FileNum;                     /* the number of NonIO file in each sub directory */
    ushort **m_FileMode;                /* mode of each NonIO file */
    int *m_DirNum;                      /* the number of directory in each sub directory */
    uchar **m_DirMode;                  /* mode of each directory, one byte (8 bits) for 8 dirs */
    int *m_SymlinkNum;                  /* the number of symlink in each sub directory */
    uchar **m_SymlinkMode;              /* mode of each symlink, one byte (8 bits) for 8 symlinks */

    _MUTEX_T m_break_lock;
    BOOL m_break_value;
	
/* constructor and destructor */
public:
    CClientFileSysBase(char *servername, char *servernetif, char *serverpath, char *localpath);
    CClientFileSysBase(char *servername, char *servernetif, char *serverpath, char *localpath,
                       int totalsize, int totalfnodenum,
                       int subdirnum, int ionum, int filenum, int dirnum, int symlinknum);
	~CClientFileSysBase();

/* operations */
public:
    void FreeAllSpace();

    char *GetServerName();
    char *GetServerNetIF();
    char *GetServerPath();
    char *GetLocalPath();

    void SetBreak(BOOL break_value=TRUE);
    BOOL IsBreak();

    void SetFileSizeDist(SFileSizeDist *dist);

    void GetPathDirName(char *name);
    void GetSubName(char *name, int subIndex);
    void GetIOFileName(char *name, int subIndex, int fileIndex);
    void GetNonIOFileName(char *name, int subIndex, int fileIndex);
    void GetDirName(char *name, int subIndex, int dirIndex);
    void GetSymlinkName(char *name, int subIndex, int dirIndex);
    void GetSymlinkTargetName(char *name);

    /* return 0:not mounted   1: mounted on the right file system   2: mounted on the wrong file system */
    void SetCached();
    BOOL IsMounted();
    BOOL CheckAndMount();
    int CheckMounted();
    BOOL Mount();
    BOOL Unmount();

    BOOL CheckWritePermission();

    char *GetHostName();
    char *GetHostIp();
    void GetID(int &job_id, int &client_id);
    BOOL GetUpdateProgress();
    void SetHostName(char *hostname);
    void SetHostIp(char *hostip);
    void SetID(int job_id, int client_id);
    void SetUpdateProgress(BOOL bUpdate);

    void Rebuild();

    void Refresh();
    void Refresh_SubDirs();
    void Refresh_Content();
    void Refresh_Content_Multithread();
    BOOL UpdateProgress(int progress, BOOL done);

    void CalFileNum();
    int RoundSubDirNum(int num);

    BOOL SendFileSysInfoToHost(CSocketClient *socket);
    BOOL SendFileSysInfoToPrim();
};

#endif

