
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientFileSysRefPool.cpp
**
** DESCRIPTION: This class mainly provide functions to manage the threads
**              to refresh file system.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
** FUNCTION LIST:
**     CClientFileSysRefPool::CClientFileSysRefPool()
**     CClientFileSysRefPool::~CClientFileSysRefPool()
**
**
****************************************************************************/


#ifndef WIN32
#include <unistd.h>
#endif
#include "Common.h"
#include "ClientFileSysRefPool.h"


CClientFileSysRefPool::CClientFileSysRefPool()
{
    m_FileSysRefNum = 0;
    m_FileSysRef = NULL;
    InitMutex(m_lock);
#ifndef WIN32
    pthread_cond_init(&m_event, NULL);
    m_IdleNum = 0;
#else
    m_hIdleSem = CreateSemaphore(NULL, 0, MAX_FILESYSREFNUM, NULL);
#endif
    m_IsIdle = NULL;
    m_NextIdleID = 0;
}

CClientFileSysRefPool::CClientFileSysRefPool(int num, CClientFileSysRefThread **FileSysRef)
{
    int i;

    m_FileSysRefNum = num;
    m_FileSysRef = new CClientFileSysRefThread * [num];
    for (i=0; i<num; i++)
        m_FileSysRef[i] = FileSysRef[i];

    InitMutex(m_lock);
#ifndef WIN32
    pthread_cond_init(&m_event, NULL);
    m_IdleNum = m_FileSysRefNum;
#else
    m_hIdleSem = CreateSemaphore(NULL, m_FileSysRefNum, MAX_FILESYSREFNUM, NULL);
#endif

    m_IsIdle = new BOOL [m_FileSysRefNum];
    for (i=0; i<num; i++)
        m_IsIdle[i] = TRUE;
    m_NextIdleID = 0;
}

CClientFileSysRefPool::~CClientFileSysRefPool()
{
#ifdef WIN32
    if (m_lock != NULL)
        CloseHandle(m_lock);
    if (m_hIdleSem != NULL)
        CloseHandle(m_hIdleSem);
#endif

    if (m_FileSysRef != NULL)
    {
        int i;
        for (i=0; i<m_FileSysRefNum; i++)
            m_FileSysRef[i]->Quit();

        _SLEEP(1);  /* wait for all threads quit completely */
//        for (i=0; i<m_FileSysRefNum; i++)
//            delete m_FileSysRef[i];
        delete [] m_FileSysRef;
    }
    if (m_IsIdle != NULL)
        delete [] m_IsIdle;
}

int CClientFileSysRefPool::GetFileSysRefThreadNum()
{
    return m_FileSysRefNum;
}

void CClientFileSysRefPool::AddFileSysRefThread(int num, CClientFileSysRefThread **FileSysRef)
{
    if (m_FileSysRef != NULL)
        delete [] m_FileSysRef;
    if (m_IsIdle != NULL)
        delete [] m_IsIdle;

    int i;
    m_FileSysRefNum = num;
    m_FileSysRef = new CClientFileSysRefThread * [num];
    for (i=0; i<num; i++)
        m_FileSysRef[i] = FileSysRef[i];

#ifndef WIN32
    m_IdleNum = m_FileSysRefNum;
#else
    ReleaseSemaphore(m_hIdleSem, m_FileSysRefNum, NULL);
#endif

    m_IsIdle = new BOOL [m_FileSysRefNum];
    for (i=0; i<num; i++)
        m_IsIdle[i] = TRUE;
    m_NextIdleID = 0;
}

int CClientFileSysRefPool::GetFileSysRefThreadID(CClientFileSysRefThread *pFileSysRef)
{
    int id;
    for (id=0; id<m_FileSysRefNum; id++)
    {
        if (m_FileSysRef[id] == pFileSysRef)
            break;
    }
    return (id == m_FileSysRefNum) ? -1 : id;
}

CClientFileSysRefThread *CClientFileSysRefPool::GetFileSysRefThreadByID(int id)
{
    return m_FileSysRef[id];
}

void CClientFileSysRefPool::WairForAllIdle()
{
#ifndef WIN32
    LockMutex(m_lock);
    while (m_IdleNum < m_FileSysRefNum)
        pthread_cond_wait(&m_event, &m_lock);
    UnlockMutex(m_lock);
#else
    for (int i=0; i<m_FileSysRefNum; i++)
        WaitForSingleObject(m_hIdleSem, INFINITE);
    ReleaseSemaphore(m_hIdleSem, m_FileSysRefNum, NULL);
#endif
}

CClientFileSysRefThread *CClientFileSysRefPool::GetIdleFileSysRefThread()
{
#ifndef WIN32
    LockMutex(m_lock);
    while (m_IdleNum == 0)
        pthread_cond_wait(&m_event, &m_lock);
    m_IdleNum --;
#else
    WaitForSingleObject(m_hIdleSem, INFINITE);
    LockMutex(m_lock);
#endif

    int i = m_NextIdleID;
    /*printf("Pool: select thread from %d -- %d %d\n", i, m_IsIdle[0], m_IsIdle[1]);*/
    while (! m_IsIdle[i])
    {
        i ++;
        if (i == m_FileSysRefNum)
            i = 0;
    }
    m_IsIdle[i] = FALSE;
    CClientFileSysRefThread *pFileSysRef = m_FileSysRef[i];
    m_NextIdleID = i + 1;
    if (m_NextIdleID == m_FileSysRefNum)
        m_NextIdleID = 0;
    /*printf("Pool: got thread %d (id=%d)\n", i, pFileSysRef->m_id);*/

    UnlockMutex(m_lock);

    return pFileSysRef;
}

void CClientFileSysRefPool::SetIdleFileSysRefThread(int id)
{
    LockMutex(m_lock);
    m_IsIdle[id] = TRUE;
#ifndef WIN32
    m_IdleNum ++;
    pthread_cond_signal(&m_event);
#else
    ReleaseSemaphore(m_hIdleSem, 1, NULL);
#endif
    UnlockMutex(m_lock);
}
