
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientFileSysThread.cpp
**
** DESCRIPTION: This class generate a thread and then build or refresh 
**              file system on the client side.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
** FUNCTION LIST:
**     CClientFileSysThread::CClientFileSysThread()
**     CClientFileSysThread::~CClientFileSysThread()
**     void CClientFileSysThread::run()
**
**
****************************************************************************/


#include <ctype.h>
#include "Common.h"
#include "ClientFileSysThread.h"
#include "ClientFileSysBase.h"
#include "ClientLoadMgrThread.h"
#include "SocketClient.h"
#include "Packet.h"
#include "NetXDT.h"
#include "Pumper.h"

extern CPumper thePumper;

CClientFileSysThread::CClientFileSysThread(CSocketClient *socket)
{
    m_socket = socket;

    m_fsbase = NULL;
}

CClientFileSysThread::~CClientFileSysThread()
{
    if (m_socket != NULL)
        delete m_socket;

    thePumper.DeleteThread(1, this);
}

void CClientFileSysThread::SetFileSysPara(char *name, char *netif, char *path, char *lpath)
{
    strcpy(m_ServerName, name);
    strcpy(m_ServerNetIF, netif);
    strcpy(m_ServerPath, path);
    strcpy(m_LocalPath, lpath);
}

void CClientFileSysThread::GetID(int &job_id, int &client_id)
{
    m_fsbase->GetID(job_id, client_id);
}

void CClientFileSysThread::BuildFileSizeDist(int fssize, SFileSizeDist &dist)
{
    dist.num = 10;
    dist.item[0].pcnt =  33;    dist.item[0].size =    1;    /* 33% - 1K */
    dist.item[1].pcnt =  54;    dist.item[1].size =    2;    /* 21% - 2K */
    dist.item[2].pcnt =  67;    dist.item[2].size =    4;    /* 13% - 4K */
    dist.item[3].pcnt =  77;    dist.item[3].size =    8;    /* 10% - 8K */
    dist.item[4].pcnt =  85;    dist.item[4].size =   16;    /*  8% - 16K */
    dist.item[5].pcnt =  90;    dist.item[5].size =   32;    /*  5% - 32K */
    dist.item[6].pcnt =  94;    dist.item[6].size =   64;    /*  4% - 64K */
	dist.item[7].pcnt =  97;    dist.item[7].size =  128;    /*  3% - 128K */
    dist.item[8].pcnt =  99;    dist.item[8].size =  256;    /*  2% - 256K */
    dist.item[9].pcnt = 100;    dist.item[9].size = 1024;    /*  1% - 1M */
                                                             /* average size = 26.71K */
    if (fssize > 0)     /* 0:small 1:medium 2:large */
    {
        int multi = (fssize == 1) ? 10 : 100;
        for (int i=0; i<10; i++)
            dist.item[i].size *= multi;
    }
}

void CClientFileSysThread::SetBreak(BOOL break_value)
{
    m_fsbase->SetBreak(break_value);
}

void CClientFileSysThread::run()
{
    printf("In CClientFileSysThread::run()\n");

    /* read message from RM */
    CPacket packet;
    char *buffer;
    int size;

    printf("Receive data of NET_MSGTYPE_PUMPER_START_REFFS ...\n");
    if (! packet.Receive(m_socket))
        return;
    buffer = packet.GetData(&size);
    printf("got %d bytes\n", size);

    /* analyze the message and ready to build file system */
    CNet_MsgInstFileSys net_msg;
    net_msg.xdt_trans(XDTOP_GET, buffer, size);

    printf("Request data is.\n");
    printf("    ServerName  = %s.\n", net_msg.ServerName);
    printf("    FileSysName = %s.\n", net_msg.FSName);
    printf("    ServerNetIF = %s.\n", net_msg.ServerNetIF);
    printf("    ServerPath  = %s.\n", net_msg.ServerPath);
    printf("    ServerShare = %s.\n", net_msg.ServerShare);

    char localPath[MAX_PATHLEN];
#ifndef WIN32
    strcpy(localPath, "");  /* will assign a path later */
#else
    sprintf(localPath, "\\\\%s\\%s", net_msg.ServerNetIF, net_msg.ServerShare);

//DEBUG
//sprintf(localPath, "c:\\Stressor\\LGFS");
#endif
    SetFileSysPara(net_msg.ServerName, net_msg.ServerNetIF, net_msg.ServerPath, localPath);

    /* build the file system */
    m_fsbase = new CClientFileSysBase(m_ServerName, m_ServerNetIF, m_ServerPath, m_LocalPath);
    m_fsbase->SetHostName(net_msg.HostName);
    if (! net_msg.UpdateProgress)   // auto refresh, from prim client
        m_fsbase->SetHostIp(net_msg.HostIp);
    else                            // user-defined refresh, from resource manager server
    {
        char *hostip = m_socket->GetFromIp();
        if (isdigit(hostip[0]))
            m_fsbase->SetHostIp(hostip);
        else
            m_fsbase->SetHostIp(net_msg.HostIp);
    }
    m_fsbase->SetID(net_msg.job_id, net_msg.client_id);
    m_fsbase->SetUpdateProgress(net_msg.UpdateProgress);
    m_fsbase->SetCached();

    /* using net_msg.FSSize to determine file set size */
    SFileSizeDist dist;
    BuildFileSizeDist(net_msg.FSSize, dist);
    m_fsbase->SetFileSizeDist(&dist);

    /* do mount operation */
    m_fsbase->CheckAndMount();

    /* for auto refresh, need tell LoadMgrThread file system was refreshed */
    if (! net_msg.UpdateProgress)
    {
        CClientLoadMgrThread *loadMgrThread = (CClientLoadMgrThread *) thePumper.FindThreadByLoadID(3, net_msg.job_id);
        if (loadMgrThread != NULL)
            loadMgrThread->SetAutoRefreshed();
    }

    /* check availability, 0:successful, 1:fail to mount 2:write permission denied */
    int status;
    if (! m_fsbase->IsMounted())
        status = 1;
    else if (! m_fsbase->CheckWritePermission())
        status = 2;
    else
        status = 0;

    /* send back response to resource manager server */
    if (net_msg.UpdateProgress)
    {
        /* 0:successful, 1:fail to mount 2:write permission denied */
        CNet_CommNum net_ack;
        net_ack.num = status;

//DEBUG
//net_ack.num = 0;
        net_ack.xdt_trans(XDTOP_PUT, buffer, size);
        packet.SetData(buffer, size);
        if (! packet.Send(m_socket))
            return;
    }

    /* don't need socket connection, so close and delete it */
    delete m_socket;
    m_socket = NULL;

//DEBUG
    if (status == 0)
    {
        printf("Refreshing the file system ...\n");
        m_fsbase->Refresh();

        /* unmount before send message back to avoid some promblems */
        m_fsbase->Unmount();
        if (! m_fsbase->IsBreak())
        {
            if (m_fsbase->GetUpdateProgress())
                m_fsbase->UpdateProgress(100, TRUE);
            else
                m_fsbase->SendFileSysInfoToPrim();
        }
    }

    delete m_fsbase;

    printf("Quit from CClientFileSysThread::run()\n");
}

