
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientLoadGenPool.h
**
** DESCRIPTION: This is header file to define the class:
**              CClientLoadGenPool
**
**              This class mainly provide functions to manage the threads
**              to generate load from client to file server.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
**  CLASS LIST:
**      CClientLoadGenPool
**
**
****************************************************************************/

#ifndef _CLIENTLOADGENPOOL_H_
#define _CLIENTLOADGENPOOL_H_

#include "ClientLoadGenThread.h"

class CClientLoadGenPool
{
private:
    CClientLoadMgrThread *m_LoadMgrThread;

    int m_LoadGenNum;
    CClientLoadGenThread **m_LoadGen;

#ifndef WIN32
    pthread_mutex_t m_lock;
    pthread_cond_t m_event;
    int m_IdleNum;
#else
    HANDLE m_lock;
    HANDLE m_hIdleSem;
#endif
    BOOL *m_IsIdle;
    int m_NextIdleID;
	
/* constructor and destructor */
public:
    CClientLoadGenPool(CClientLoadMgrThread *pLoadMgrThread);
    CClientLoadGenPool(int num, CClientLoadGenThread **LoadGen, CClientLoadMgrThread *pLoadMgrThread);
	~CClientLoadGenPool();

/* operations */
public:
    int GetLoadGenThreadNum();

    void AddLoadGenThread(int num, CClientLoadGenThread **LoadGen);

    void WairForAllIdle();
    int GetLoadGenThreadID(CClientLoadGenThread *pLoadGen);
    CClientLoadGenThread *GetLoadGenThreadByID(int id);

    CClientLoadGenThread *GetIdleLoadGenThread();
    void SetIdleLoadGenThread(int id);
};

/*extern CClientLoadGenPool theLoadGenPool;*/

#endif

