
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientOperation.cpp
**
** DESCRIPTION: These classes provide the functions to execute an operation.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
** FUNCTION LIST:
**     CClientOperation::CClientOperation()
**     CClientOperation::~CClientOperation()
**
**     CClientOprtRead::CClientOprtRead()
**     CClientOprtRead::~CClientOprtRead()
**
**     CClientOprtWrite::CClientOprtWrite()
**     CClientOprtWrite::~CClientOprtWrite()
**
**     CClientOprtGetAttr::CClientOprtGetAttr()
**     CClientOprtGetAttr::~CClientOprtGetAttr()
**
**
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "Common.h"
#include "ClientOperation.h"
#include "ClientLoadMgrThread.h"


static CClientOprtRead      op0;
static CClientOprtOverWrite op1;
static CClientOprtAppend    op2;
static CClientOprtTruncate  op3;
static CClientOprtStat      op4;
static CClientOprtChmod     op5;
static CClientOprtCreatNULL op6;
static CClientOprtMkdir     op7;
static CClientOprtRmdir     op8;
static CClientOprtReadDir   op9;        
static CClientOprtSymlink   op10;
static CClientOprtReadlink  op11;
static CClientOprtLink      op12;
static CClientOprtUnlink    op13;
static CClientOprtRename    op14;
static CClientOprtStatvfs   op15;
static CClientOprtMemmap    op16;

Op_Map Ops_Pcnt[MAX_OPNUM+1] = {

/* Op Class    mix   call   req  req  req   op class
    name       pcnt  targ   pcnt cnt  targ    ptr       */

 { "Read",      20,  0,    0,    0,   0,    &op0  },
 { "OverWrite", 20,  0,    0,    0,   0,    &op1  },
 { "Append",     7,  0,    0,    0,   0,    &op2  },
 { "Truncate",   4,  0,    0,    0,   0,    &op3  },
 { "Stat",       5,  0,    0,    0,   0,    &op4  },
 { "Chmod",      5,  0,    0,    0,   0,    &op5  },
 { "CreatNULL",  5,  0,    0,    0,   0,    &op6  },
 { "Mkdir",      5,  0,    0,    0,   0,    &op7  },
 { "Rmdir",      5,  0,    0,    0,   0,    &op8  },
 { "ReadDir",    5,  0,    0,    0,   0,    &op9  },
 { "Symlink",    2,  0,    0,    0,   0,    &op10 },
 { "Readlink",   2,  0,    0,    0,   0,    &op11 },
 { "Link",       2,  0,    0,    0,   0,    &op12 },
 { "Unlink",     2,  0,    0,    0,   0,    &op13 },
 { "Rename",     5,  0,    0,    0,   0,    &op14 },
 { "Statvfs",    5,  0,    0,    0,   0,    &op15 },
 { "Memmap",     1,  0,    0,    0,   0,    &op16 },
 { "TOTAL",               100,  0,    0,    0,   0,    NULL  }
};


/************************************************************/
/*           CClientOperation : the base class              */
/************************************************************/
CClientOperation::CClientOperation()
{
    load_mgr_ptr = NULL;
}

CClientOperation::CClientOperation(CClientLoadMgrThread *host)
{
    load_mgr_ptr = host;
}

CClientOperation::~CClientOperation()
{
}

CClientOperation *CClientOperation::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOperation(host);
}


BOOL CClientOperation::DoOP()
{
    return TRUE;
}

/************************************************************/
/*                    CClientOprtRead                       */
/************************************************************/
CClientOprtRead::CClientOprtRead() : CClientOperation()
{
}

CClientOprtRead::CClientOprtRead(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRead::~CClientOprtRead()
{

}

CClientOperation * CClientOprtRead::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRead(host);
}


BOOL CClientOprtRead::DoOP()
{
    int   syscall_ret;      /* result from system call */
    int	  cur_cnt;          /* packet loop variable */
    int	  max_cnt;	        /* packet number */
    int   offset;           /* offset position reading from */
    int	  size;             /* data size each reading */
    int   frag_size;        /* data size of the fragment */
    int   total_size;       /* data size read */
   

    /* get transfer size for read operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(1);

    /* Don't allow a read of less than one block size */
    //Bytes_per_block = kb_per_block * 1024;
    if (xfer_size < Bytes_per_block)
        xfer_size = Bytes_per_block;

    /* pick a file for read operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do read\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);

    /* randomly choose an offset that is a multiple of the block size
     * and constrained by making the transfer fit within the file */
    offset = 0;
    if (file_size > xfer_size)
        offset = Bytes_per_block * (GetRandom() % (((file_size - xfer_size) / Bytes_per_block) + 1));

    _ASSERT(file_size >= offset + xfer_size);
    
//    printf("Read file (%s, size=%06x) %06x bytes (from %06x to %06x)\n", file_name, file_size, xfer_size, offset, offset+xfer_size);
    
    /* open the file for reading */
    file_handle = open(file_name, O_RDONLY);
    if (file_handle < 0)
    {
        fprintf(stderr, "Read: open error on file %s (%s)\n", file_name, strerror(errno));
        syscall_ret = -1;
    }
    else
    {
        syscall_ret = lseek(file_handle, offset, SEEK_SET);
        if (syscall_ret < 0)
        {
            fprintf(stderr, "Read: lseek error on file %s (%s)\n", file_name, strerror(errno));
            close(file_handle);
        }
    }
        
    if (syscall_ret < 0)
    {
        /* must call SetIOFileInfo to release this file before return */
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }
        
    /* start to read. read the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;
    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;
    
    if (frag_size > 0)
        max_cnt ++;

    total_size = 0;

    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should read fragment */
        if (total_size > xfer_size)
            size = frag_size;

        syscall_ret = read(file_handle, buf, size);
        if (syscall_ret < 0)
        {
            fprintf(stderr, "Read: read error on file %s (%s)\n", file_name, strerror(errno));
            break;
        }
    } /* for reading max_cnt packets */

    /* close the file descriptor */
    close(file_handle);

    /* set back the file size to release this file */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);

    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}


/************************************************************/
/*                    CClientOprtOverWrite                  */
/************************************************************/
unsigned int CClientOprtOverWrite::m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

CClientOprtOverWrite::CClientOprtOverWrite() : CClientOperation()
{
}

CClientOprtOverWrite::CClientOprtOverWrite(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtOverWrite::~CClientOprtOverWrite()
{
}

void CClientOprtOverWrite::InitWriteBuffer()
{
    unsigned int *bp;
    unsigned int *be  = m_WriteBuf + (sizeof(m_WriteBuf) / sizeof(unsigned int));

    for (bp = m_WriteBuf; bp < be; bp++)
        *bp = (unsigned int)0xdeadbeef;
}

CClientOperation * CClientOprtOverWrite::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtOverWrite(host);
}


BOOL CClientOprtOverWrite::DoOP()
{
    
    int syscall_ret;            /* result from system call */
    int cur_cnt;                /* packet loop variable */
    int max_cnt;	            /* packet number */
    int offset;                 /* offset position writing from */
    int	size;                   /* data size each writing */
    int frag_size;              /* data size of the fragment */
    int total_size;             /* data size written */

    /* get transfer size for overwrite operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* Don't allow a read of less than one block size */
    if (xfer_size < Bytes_per_block)
    	xfer_size = Bytes_per_block;

    /* pick a file for overwrite operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do overwrite\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);
        
    /* if the transfer data size large than 128 blocks, then set the flush flag */
    BOOL FlushFlag = FALSE;
    if (xfer_size > Bytes_per_block * 128)
        FlushFlag = TRUE;

    /* randomly choose an offset that is a multiple of the block size
     * and constrained by making the transfer fit within the file */
    offset = 0;
    if (file_size > xfer_size)
        offset = Bytes_per_block * (GetRandom() % (((file_size - xfer_size) / Bytes_per_block) + 1));
   
    _ASSERT(file_size >= offset + xfer_size);

    /* open the file for writing */
    file_handle = open(file_name, O_WRONLY);
    if (file_handle < 0)
        syscall_ret = -1;
    else
    {
        syscall_ret = lseek(file_handle, offset, SEEK_SET);
        if (syscall_ret < 0)
            close(file_handle);
    }
    
    if (syscall_ret < 0)
    {
        /* must call SetIOFileInfo to release this file before return */
        fprintf(stderr, "overwrite call error on file %s\n", file_name);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
	    return FALSE;
    }    

    /* start to write. write the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;

    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;

    if (frag_size > 0)
        max_cnt ++;
    
    total_size = 0;
    
    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should write fragment */
        if (total_size > xfer_size)
            size = frag_size;
 
        syscall_ret = write(file_handle, m_WriteBuf, size);

        if (syscall_ret < 0)
        {
            fprintf(stderr, "overwrite call error on file %s\n", file_name);
            break;
        } 
    } /* for writing max_cnt packets */

    if (FlushFlag)
    {
        FILE *fp = fdopen(file_handle, "wb");
        if (fp != NULL)
        {
            fflush(fp);
            fclose(fp);
        }
    }
    
    close(file_handle);

    /* set back the file size to release this file */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);

    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}

/************************************************************/
/*                   CClientOprtAppend                      */
/************************************************************/
unsigned int CClientOprtAppend::m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

CClientOprtAppend::CClientOprtAppend() : CClientOperation()
{
}

CClientOprtAppend::CClientOprtAppend(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtAppend::~CClientOprtAppend()
{
}

CClientOperation * CClientOprtAppend::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtAppend(host);
}


void CClientOprtAppend::InitWriteBuffer()
{
    unsigned int *bp;
    unsigned int *be  = m_WriteBuf + (sizeof(m_WriteBuf) / sizeof(unsigned int));

    for (bp = m_WriteBuf; bp < be; bp++)
        *bp = (unsigned int)0xdeadbeef;
}

BOOL CClientOprtAppend::DoOP()
{
    int             syscall_ret;               /* result from system call */
    int			    cur_cnt;                   /* packet loop variable */
    int			    max_cnt;	               /* packet number */
    int             offset;                    /* offset position writing from */
    int		    	size;                      /* data size each writing */
    int             frag_size;                 /* data size of the fragment */
    int             total_size;                /* data size written */

    /* get transfer size for append operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* Don't allow a read of less than one block size */
    if (xfer_size < Bytes_per_block)
	    xfer_size = Bytes_per_block;

    /* pick a file for append operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(0, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do append\n");
        return FALSE;
    }
    
    /* if the transfer data size large than 128 blocks, then set the flush flag */
    BOOL FlushFlag = FALSE;
    if (xfer_size > Bytes_per_block * 128)
        FlushFlag = TRUE;

    /* open the file for appending */
    file_handle = open(file_name, O_WRONLY | O_APPEND);
    if (file_handle  < 0)
    {
        /* must call SetIOFileInfo to release this file before return */
        fprintf(stderr, "append call error on file %s: %s\n", file_name, strerror(errno));
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }    

    /* start to write. write the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;

    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;

    if (frag_size > 0)
        max_cnt ++;
    
    total_size = 0;
    
    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should write fragment */
        if (total_size > xfer_size)
            size = frag_size;

        syscall_ret = write(file_handle, m_WriteBuf, size);

        if (syscall_ret < 0)
        {
            fprintf(stderr, "append call error on file %s: %s\n", file_name, strerror(errno));
            break;
        } 
    } /* for writing max_cnt packets */

    if (FlushFlag)
    {
        FILE *fp = fdopen(file_handle, "wb");
        if (fp != NULL)
        {
            fflush(fp);
            fclose(fp);
        }
    }

    close(file_handle);

    /* set the new file size */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size + xfer_size);
    
    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}

/************************************************************/
/*                   CClientOprtTruncate                    */
/************************************************************/
CClientOprtTruncate::CClientOprtTruncate() : CClientOperation()
{
}

CClientOprtTruncate::CClientOprtTruncate(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtTruncate::~CClientOprtTruncate()
{
}

CClientOperation * CClientOprtTruncate::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtTruncate(host);
}


BOOL CClientOprtTruncate::DoOP()
{
    BOOL            ret;

    /* get truncating size for truncate operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* pick a file for truncate operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do truncate\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);

    /* make the call */
    if (truncate(file_name, file_size - xfer_size) >= 0)
        ret = TRUE;
    else
    {
        fprintf(stderr, "truncate call error on file %s\n", file_name);
        ret = FALSE;
    } 

    /* set the new file size */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size - xfer_size);

    load_mgr_ptr->AddOps(1);

    return ret;
}


/************************************************************/
/*                   CClientOprtStat                        */
/************************************************************/
CClientOprtStat::CClientOprtStat() : CClientOperation()
{
}

CClientOprtStat::CClientOprtStat(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

/* option: 0(access) 1(remove) 2(create) */
CClientOprtStat::~CClientOprtStat()
{
}

CClientOperation * CClientOprtStat::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtStat(host);
}

BOOL CClientOprtStat::DoOP()
{
    struct stat         statinfo;       /* the stat of this file */
    BOOL                ret;

    /* pick a file for stat operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do stat\n");
        return FALSE;
    }

    /* make the call */
    if (stat(file_name, &statinfo) == 0)
        ret = TRUE;
    else
    {
	    fprintf(stderr, "stat call error on file %s\n", file_name);
	    ret = FALSE;
    } 

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtChmod                       */
/************************************************************/
CClientOprtChmod::CClientOprtChmod() : CClientOperation()
{
}

CClientOprtChmod::CClientOprtChmod(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtChmod::~CClientOprtChmod()
{
}

CClientOperation * CClientOprtChmod::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtChmod(host);
}

BOOL CClientOprtChmod::DoOP()
{
    mode_t new_mode;       /* file new mode */
    BOOL ret;

    /* pick a file for chmod operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do chmod\n");
        return FALSE;
    }

    /* make the call */
    new_mode = mode ^ 00077;
    if (chmod(file_name, new_mode) == 0)
        ret = TRUE;
	else
    {
	    fprintf(stderr, "chmod call error on file %s\n", file_name);
        ret = FALSE;
    } 
    
    /* set the new file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, new_mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtCreatNULL                   */
/************************************************************/
CClientOprtCreatNULL::CClientOprtCreatNULL() : CClientOperation()
{
}

CClientOprtCreatNULL::CClientOprtCreatNULL(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtCreatNULL::~CClientOprtCreatNULL()
{
}

CClientOperation * CClientOprtCreatNULL::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtCreatNULL(host);
}

BOOL CClientOprtCreatNULL::DoOP()
{
    BOOL ret;
    
    /* pick a file for create operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(2, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do create\n");
        return FALSE;
    }

    /* make the call */
    mode = 00666;
    file_handle = creat(file_name, mode);

    if (file_handle < 0)
    {
        fprintf(stderr, "create call error on file %s: %s\n", file_name, strerror(errno));
        ret = FALSE;
    }
    else
    {
        close(file_handle);
        ret = TRUE;
    }

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(2, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtMkdir                       */
/************************************************************/
CClientOprtMkdir::CClientOprtMkdir() : CClientOperation()
{
}

CClientOprtMkdir::CClientOprtMkdir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtMkdir::~CClientOprtMkdir()
{
}

CClientOperation * CClientOprtMkdir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtMkdir(host);
}

BOOL CClientOprtMkdir::DoOP()
{
    BOOL ret;

    /* pick a dir for mkdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(2, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do mkdir\n");
        return FALSE;
    }

    /* make the call */
    if (mkdir(dir_name, 00777) == 0) 
        ret = TRUE;		
    else 
    {
        fprintf(stderr, "Mkdir: mkdir error on dir %s (%s)\n", dir_name, strerror(errno));
        ret = FALSE;
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(2, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtRmdir                       */
/************************************************************/
CClientOprtRmdir::CClientOprtRmdir() : CClientOperation()
{
}

CClientOprtRmdir::CClientOprtRmdir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRmdir::~CClientOprtRmdir()
{
}


CClientOperation * CClientOprtRmdir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRmdir(host);
}

BOOL CClientOprtRmdir::DoOP()
{
    BOOL ret;   
    
    /* pick a dir for rmdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(1, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do rmdir\n");
        return FALSE;
    }

    /* make the call */
    if (rmdir(dir_name) == 0) 
		ret = TRUE;
    else 
    {
        fprintf(stderr, "Rmdir: rmdir error on dir %s (%s)\n", dir_name, strerror(errno));
        ret = FALSE;
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(1, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtReadDir                     */
/************************************************************/
CClientOprtReadDir::CClientOprtReadDir() : CClientOperation()
{
}

CClientOprtReadDir::CClientOprtReadDir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtReadDir::~CClientOprtReadDir()
{
}

CClientOperation * CClientOprtReadDir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtReadDir(host);
}

BOOL CClientOprtReadDir::DoOP()
{
    DIR *dirp;          /* to read a directory */
    BOOL ret;

    /* pick a dir for readdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(0, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do readdir\n");
        return FALSE;
    }

    /* make the call */
    dirp = opendir(dir_name);
    if (dirp == NULL)
    {
        fprintf(stderr, "Readdir call error on dir %s\n", dir_name);
        ret = FALSE;
    }
    else
    {
        while (readdir(dirp))
            ;
        closedir(dirp);
        ret = TRUE;
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(0, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtSymlink                     */
/************************************************************/
CClientOprtSymlink::CClientOprtSymlink() : CClientOperation()
{
}

CClientOprtSymlink::CClientOprtSymlink(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtSymlink::~CClientOprtSymlink()
{
}

CClientOperation * CClientOprtSymlink::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtSymlink(host);
}

BOOL CClientOprtSymlink::DoOP()
{
    BOOL ret;

    /* pick a symlink for symlink operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetSymlinkInfo(2, file_name, fs, fd))
    {
        fprintf(stderr, "Can't find a symlink to do symlink\n");
        return FALSE;
    }

    /* put a random string into target_file_name */
    int value = GetRandom() % 100;
    if (value < 33)
        strcpy(target_file_name, "/DIR1/DIR2/FILE");
    else if (value < 66)
        strcpy(target_file_name, "../../DIR/FILE");
    else
        strcpy(target_file_name, "DIR1/DIR2/FILE");

    /* make the call */
    if (symlink(target_file_name, file_name) != -1) 
        ret = TRUE;
    else 
    {
        fprintf(stderr, "symlink call error on file %s and target file %s\n", file_name, target_file_name);
        ret = FALSE;
    }

    /* set back the symlink mode to release this symlink */
    load_mgr_ptr->GetFileSysManager()->SetSymlinkInfo(2, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtReadlink                    */
/************************************************************/
CClientOprtReadlink::CClientOprtReadlink() : CClientOperation()
{
}

CClientOprtReadlink::CClientOprtReadlink(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtReadlink::~CClientOprtReadlink()
{
}

CClientOperation * CClientOprtReadlink::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtReadlink(host);
}

BOOL CClientOprtReadlink::DoOP()
{
    char sym_data[MAX_PATHLEN];	
    BOOL ret;

    /* pick a symlink for readlink operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetSymlinkInfo(0, file_name, fs, fd))
    {
        fprintf(stderr, "Can't find a symlink to do readlink\n");
        return FALSE;
    }

    /* make the call */
    if (readlink(file_name, sym_data, sizeof(sym_data)) != -1)
        ret = TRUE;
    else 
    {
        fprintf(stderr, "readlink call error on symlink %s\n", file_name);
        ret = FALSE;
    }

    /* set back the symlink mode to release this symlink */
    load_mgr_ptr->GetFileSysManager()->SetSymlinkInfo(0, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtLink                        */
/************************************************************/
CClientOprtLink::CClientOprtLink() : CClientOperation()
{
}

CClientOprtLink::CClientOprtLink(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtLink::~CClientOprtLink()
{
}

CClientOperation * CClientOprtLink::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtLink(host);
}

BOOL CClientOprtLink::DoOP()
{
    BOOL ret;
    int fs2, fd2, mode2;

    /* pick two files for link operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, target_file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find an existing file to do link\n");
        return FALSE;
    }
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(2, file_name, fs2, fd2, mode2, fs))
    {
        /* must call SetNonIOFileInfo to release this file before return */
        load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);
        fprintf(stderr, "Can't find a new file to do link\n");
        return FALSE;
    }
    mode2 = mode;

    /* make the call */
    if (link(target_file_name, file_name) != -1) 
        ret = TRUE;
    else 
    {
        fprintf(stderr, "link call error on file %s and target file %s\n", file_name, target_file_name);
        ret = FALSE;
    }

    /* set back the two files' mode to release these files */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(2, fs2, fd2, mode2);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtUnlink                      */
/************************************************************/
CClientOprtUnlink::CClientOprtUnlink() : CClientOperation()
{
}

CClientOprtUnlink::CClientOprtUnlink(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtUnlink::~CClientOprtUnlink()
{
}


CClientOperation * CClientOprtUnlink::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtUnlink(host);
}

BOOL CClientOprtUnlink::DoOP()
{
    BOOL ret;

    /* pick a file for unlink operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(1, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do unlink\n");
        return FALSE;
    }

    /* make the call */
    if (unlink(file_name) != -1) 
        ret = TRUE;
    else 
    {
        fprintf(stderr, "link call error on file %s\n", file_name);
        ret = FALSE;
    }

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(1, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtRename                      */
/************************************************************/
CClientOprtRename::CClientOprtRename() : CClientOperation()
{
}

CClientOprtRename::CClientOprtRename(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRename::~CClientOprtRename()
{
}


CClientOperation * CClientOprtRename::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRename(host);
}

BOOL CClientOprtRename::DoOP()
{
    BOOL ret;
    int fs2, fd2, mode2;

    /* pick two files for rename operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(1, target_file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find an existing file to do rename\n");
        return FALSE;
    }
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(2, file_name, fs2, fd2, mode2, fs))
    {
        /* we didn't delete this file, so release it using access option */
        load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);
        fprintf(stderr, "Can't find a new file to do rename\n");
        return FALSE;
    }
    mode2 = mode;

    /* make the call */
    if (rename(target_file_name, file_name) != -1) 
        ret = TRUE;
    else 
    {
        fprintf(stderr, "rename call error on file %s\n", file_name);
        ret = FALSE;
    }

    /* set back the two files' mode to release these files */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(1, fs, fd, mode);
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(2, fs2, fd2, mode2);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtStatvfs                     */
/************************************************************/
CClientOprtStatvfs::CClientOprtStatvfs() : CClientOperation()
{
}

CClientOprtStatvfs::CClientOprtStatvfs(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtStatvfs::~CClientOprtStatvfs()
{
}


CClientOperation * CClientOprtStatvfs::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtStatvfs(host);
}

BOOL CClientOprtStatvfs::DoOP()
{
    BOOL ret;

    /* pick a file for statvfs operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do statvfs\n");
        return FALSE;
    }

    /* make the call */
    if (statvfs(file_name, &vfsinfo) != -1) 
        ret = TRUE;
    else 
    {
        fprintf(stderr, "Statvfs call error on file %s\n", file_name);
        ret = FALSE;
    }

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtMemmap                      */
/************************************************************/

CClientOprtMemmap::CClientOprtMemmap() : CClientOperation()
{
}

CClientOprtMemmap::CClientOprtMemmap(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtMemmap::~CClientOprtMemmap()
{
    
}


CClientOperation * CClientOprtMemmap::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtMemmap(host);
}
   

BOOL CClientOprtMemmap::DoOP()
{
    char            *buffer;
    int             i;
    BOOL            ret;

    /* get transfer size for mmap operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* Don't allow a read of less than one block size */
    if (xfer_size < Bytes_per_block)
        xfer_size = Bytes_per_block;

    /* pick a file for mmap operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do memmap\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);

    /* get the number of pages and discard the remainder */
    pagesize = sysconf(_SC_PAGESIZE);
	if (pagesize < 0)
    {
		/* fprintf(stderr, "can't get page size\n"); */
        pagesize = 8192;
    }
    numpages = xfer_size / pagesize;
    xfer_size = numpages * pagesize;

    /* open the file with write option */
    if ((file_handle = open(file_name, O_RDWR)) == -1)
    {
        /* must call SetIOFileInfo to release this file before return */
	    fprintf(stderr, "can't open file %s for mapping: %s\n", file_name, strerror(errno));
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    } 

    /* map the file with memory */
    buffer = mmap(0, xfer_size, PROT_READ | PROT_WRITE, MAP_SHARED, file_handle, 0);
    if (buffer == (char *)MAP_FAILED)
    {
        /* must call SetIOFileInfo to release this file before return */
	    fprintf(stderr, "can't map %s for writing: %s\n", file_name, strerror(errno));
	    close(file_handle);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    } 

    /* fill the memory */
	unsigned char val;
    for (i = 0; i < numpages; i++)
    {
        val = 'a' + (i % 26);
//        printf("fill the page #%d of size %d\n", i, pagesize);
		memset(buffer + i * pagesize, val, pagesize);
	}

    /* write into disk */
    if (msync(buffer, xfer_size, MS_SYNC | MS_INVALIDATE) < 0)
    {
        /* must call SetIOFileInfo to release this file before return */
        fprintf(stderr, "can't msync %s: %s\n", file_name, strerror(errno));
        close(file_handle);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }

    /* unmap */
    if (munmap(buffer, xfer_size) < 0) 
    {
		fprintf(stderr, "can't munmap %s: %s\n", file_name, strerror(errno));
        ret = FALSE;
	}
    else
        ret = TRUE;
    
    close(file_handle);

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);

    load_mgr_ptr->AddOps(1);

    return ret;
}
