
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientOperation.h
**
** DESCRIPTION: This is header file to define the class:
**                  CClientOperation
**              and its subclasses:
**                  CClientOprtRead
**                  CClientOprtWrite
**                  CClientOprtGetAttr
**                    ... ...
**
**              These classes implememt the opreations for load generation.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
**  CLASS LIST:
**      CClientOperation
**      CClientOprtRead
**      CClientOprtOverWrite
**      CClientOprtAppend
**      CClientOprtTruncate
**      CClientOprtStat
**      CClientOprtChmod
**	    CClientOprtCreatNULL
**	    CClientOprtMkdir
**	    CClientOprtRmdir
**	    CClientOprtReadDir
**	    CClientOprtSymlink
**	    CClientOprtReadlink
**	    CClientOprtLink
**	    CClientOprtUnlink
**	    CClientOprtRename
**	    CClientOprtStatvfs
**	    CClientOprtMemmap
**	    CClientOprtMemunmap
**	    
**
****************************************************************************/

#ifndef _CCLIENTOPERATION_H_
#define _CCLIENTOPERATION_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statvfs.h>
#include "Common.h"
#include "SysTime.h"

class CClientLoadMgrThread;

#define Bytes_per_block  8192
#define MAX_WRTBUFSIZE   32768

/************************************************************/
/*           CClientOperation : the base class              */
/************************************************************/
class CClientOperation
{
private:

protected:
    int     xfer_size;
    int     fs;
    int     fd;
    int     file_handle;
    int     mode;
    char    file_name[MAX_PATHLEN];
    char    target_file_name[MAX_PATHLEN];
    char    dir_name[MAX_PATHLEN];
    char    target_dir_name[MAX_PATHLEN];
    int     file_size;
    char    buf[MAX_BUFSIZE]; /* data buffer */
    int     kb_per_block;
    int     good_calls;
    int     bad_calls;
    CClientLoadMgrThread *load_mgr_ptr;

/* constructor and destructor */
public:
    CClientOperation();
    CClientOperation(CClientLoadMgrThread *host);
	~CClientOperation();

/* operations */
public:
    virtual CClientOperation *DupObj(CClientLoadMgrThread *host);
    virtual BOOL DoOP();
    
};


/************************************************************/
/*                    CClientOprtRead                       */
/************************************************************/
class CClientOprtRead : public CClientOperation
{
private:
	
    /* constructor and destructor */
public:
    CClientOprtRead();
    CClientOprtRead(CClientLoadMgrThread *host);
	~CClientOprtRead();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};


/************************************************************/
/*                    CClientOprtOverWrite                      */
/************************************************************/
class CClientOprtOverWrite : public CClientOperation
{
private:
    static unsigned int m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

/* constructor and destructor */
public:
    CClientOprtOverWrite();
    CClientOprtOverWrite(CClientLoadMgrThread *host);
	~CClientOprtOverWrite();
    
/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);

    static void InitWriteBuffer();

    BOOL  DoOP();
};

/************************************************************/
/*                    CClientOprtAppend                      */
/************************************************************/
class CClientOprtAppend : public CClientOperation
{
private:
    static unsigned int m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

/* constructor and destructor */
public:
    CClientOprtAppend();
    CClientOprtAppend(CClientLoadMgrThread *host);
	~CClientOprtAppend();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    static void InitWriteBuffer();
    BOOL  DoOP();
};

/************************************************************/
/*                    CClientOprtTruncate                      */
/************************************************************/
class CClientOprtTruncate : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtTruncate();
    CClientOprtTruncate(CClientLoadMgrThread *host);
	~CClientOprtTruncate();

/* operations */
public:
   CClientOperation *DupObj(CClientLoadMgrThread *host);    
   BOOL  DoOP();

};

/************************************************************/
/*                   CClientOprtStat                     */
/************************************************************/
class CClientOprtStat : public CClientOperation
{
private:
   
/* constructor and destructor */
public:
    CClientOprtStat();
    CClientOprtStat(CClientLoadMgrThread *host);
	~CClientOprtStat();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtChmod                     */
/************************************************************/
class CClientOprtChmod : public CClientOperation
{
private:
	struct stat statinfo;
    
/* constructor and destructor */
public:
    CClientOprtChmod();
    CClientOprtChmod(CClientLoadMgrThread *host);
	~CClientOprtChmod();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);    
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtCreatNULL                   */
/************************************************************/
class CClientOprtCreatNULL : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtCreatNULL();
    CClientOprtCreatNULL(CClientLoadMgrThread *host);
	~CClientOprtCreatNULL();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);    
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtMkdir                     */
/************************************************************/
class CClientOprtMkdir : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtMkdir();
    CClientOprtMkdir(CClientLoadMgrThread *host);
	~CClientOprtMkdir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtRmdir                       */
/************************************************************/
class CClientOprtRmdir : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtRmdir();
    CClientOprtRmdir(CClientLoadMgrThread *host);
	~CClientOprtRmdir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtReadDir                     */
/************************************************************/
class CClientOprtReadDir : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtReadDir();
    CClientOprtReadDir(CClientLoadMgrThread *host);
	~CClientOprtReadDir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtSymlink                     */
/************************************************************/
class CClientOprtSymlink : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtSymlink();
    CClientOprtSymlink(CClientLoadMgrThread *host);
	~CClientOprtSymlink();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtReadlink                    */
/************************************************************/
class CClientOprtReadlink : public CClientOperation
{
private:

/* constructor and destructor */
public:
    CClientOprtReadlink();
    CClientOprtReadlink(CClientLoadMgrThread *host);
	~CClientOprtReadlink();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtLink                     */
/************************************************************/
class CClientOprtLink : public CClientOperation
{
private:
   
   
/* constructor and destructor */
public:

    CClientOprtLink();
    CClientOprtLink(CClientLoadMgrThread *host);
	~CClientOprtLink();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtUnlink                     */
/************************************************************/
class CClientOprtUnlink : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtUnlink();
    CClientOprtUnlink(CClientLoadMgrThread *host);
	~CClientOprtUnlink();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtRename                     */
/************************************************************/
class CClientOprtRename : public CClientOperation
{
private:
	
/* constructor and destructor */
public:
    CClientOprtRename();
    CClientOprtRename(CClientLoadMgrThread *host);
	~CClientOprtRename();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtStatvfs                     */
/************************************************************/
class CClientOprtStatvfs : public CClientOperation
{
private:
	struct statvfs      vfsinfo;        /* the infomation of file system */
/* constructor and destructor */
public:
    CClientOprtStatvfs();
    CClientOprtStatvfs(CClientLoadMgrThread *host);
	~CClientOprtStatvfs();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtMemmap                     */
/************************************************************/
class CClientOprtMemmap : public CClientOperation
{
private:
    int  pagesize;	
    int  numpages;

/* constructor and destructor */
public:
    CClientOprtMemmap();
    CClientOprtMemmap(CClientLoadMgrThread *host);
	~CClientOprtMemmap();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*  define constant for each operation */

#define OPRT_READ           0
#define OPRT_OVERWRITE      1
#define OPRT_APPEND         2
#define OPRT_TRUNCATE       3   
#define OPRT_STAT           4
#define OPRT_CHMOD          5
#define OPRT_CREATENULL     6
#define OPRT_MKDIR          7
#define OPRT_RMDIR          8
#define OPRT_READDIR        9
#define OPRT_SYMLINK        10
#define OPRT_READLINK       11
#define OPRT_LINK           12
#define OPRT_UNLINK         13
#define OPRT_RENAME         14
#define OPRT_STATVFS        15
#define OPRT_MEMMAP         16
#define TOTAL               17      /* slot for totals */

#define MAX_OPNUM           TOTAL

struct Op_Map
{
    char    op_name[MAX_NAMELEN];   /* operation name */
    int     op_pcnt;                /* percentage of call target */  
    int	    target_calls;	        /* number of calls to make */
    double	req_pcnt;	            /* cumulative request percentile */
	int		req_cnt;	            /* number of requests made */
	int		target_reqs;	        /* number of req to be made */
    CClientOperation *op;           /* an instance of each operation class */
};

extern Op_Map Ops_Pcnt[MAX_OPNUM+1];

#endif
