
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientPrimMgrThread.h
**
** DESCRIPTION: This is header file to define the class:
**              CClientPrimMgrThread
**
**              This class mainly generate a thread to do the primary client
**              work.
**              
**
** AUTHOR: Yubing Wang       Mar-02-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
**  CLASS LIST:
**      CClientPrimMgrThread
**
**
****************************************************************************/

#ifndef _CLIENTPRIMMGRTHREAD_H_
#define _CLIENTPRIMMGRTHREAD_H_

#include "Common.h"
#include "BaseThread.h"
#include "SocketClient.h"
#include "NetXDT.h"
#include "ConfigFile.h"
#include "FSInfoFile.h"

#define MAX_CLIENT_NUM      64          /* max number of client for one load */

struct SPrimFileSysInfo
{
    char ServerName[MAX_NAMELEN];       /* server name, for example copper:server_2 */
    char FsName[MAX_NAMELEN];           /* file system name */
    char ServerNetIF[MAX_NAMELEN];      /* server network interface name or ip */
    char ServerPath[MAX_PATHLEN];       /* server: the path that file system mount on */
    char ShareName[MAX_NAMELEN];        /* server: the shared name of file system for CIFS protocol */
    int FsSize;                         /* file set size, 0 small 1 medium 2 large */

    int TotalSize;                      /* file system size (max is 2048GB) in KBytes */
    int TotalFNodeNum;                  /* total number of file nodes (inodes) */
//    SFileSizeDist IODist;               /* file size distribution for IO operations */
    int SubDirNum;                      /* files are put into different subdirectory, this is dir number */
    int TotalIONum;                     /* total file number to do IO operations */
    int TotalFileNum;                   /* total file number to do NonIO operations */
    int TotalDirNum;                    /* total directory number to do directory operations */
    int TotalSymlinkNum;                /* total symlink number to do symlink operations */

    int *IONum;
    int **IOSize;
    int *FileNum;
    ushort **FileMode;
    int *DirNum;
    uchar **DirMode;
    int *SymlinkNum;
    uchar **SymlinkMode;

    int *OptDir;                        /* sub directory index (shuffled) */
    int OptDirNum[MAX_CLIENT_NUM];      /* sub directory number of each client */
};

class CClientPrimMgrThread : public CBaseThread
{
private:
    CSocketClient *m_socket;

    char m_rmname[MAX_NAMELEN];         /* machine name of resource manager server */
    char m_rmip[MAX_IPADDRLEN];         /* ip address of resource manager server */
    char m_myname[MAX_NAMELEN];         /* host name of this client */
    char m_myip[MAX_IPADDRLEN];         /* ip address of this client */

    int m_load_id;                      /* identity of this load */

    int m_ref_client_num;               /* number of clients refreshing file system */

    int m_client_num;
    char m_clients[MAX_CLIENT_NUM][MAX_NAMELEN];
    char m_client_ips[MAX_CLIENT_NUM][MAX_IPADDRLEN];
    int m_load[MAX_CLIENT_NUM];
    BOOL m_stress_server;

    int m_status;                       /* 0:Initializing 1:Running 2:Refreshing */
    int m_run_time;
    int m_used_time;

    int m_shuffle_time;

    int m_fsnum;
    SPrimFileSysInfo *m_fs[MAX_FSNUM]; 

#ifndef WIN32
    pthread_mutex_t m_ra_lock;
    pthread_cond_t m_ra_event;
    BOOL m_ra_ready;

    pthread_mutex_t m_lc_lock;
    pthread_cond_t m_lc_event;
    int m_lc_ready;
    int m_lc_status[MAX_CLIENT_NUM];

    pthread_mutex_t m_ref_lock;
    pthread_cond_t m_ref_event;
    int m_ref_ready;
    int m_ref_signal;

    pthread_mutex_t m_signal_lock;
    pthread_cond_t m_signal_event;
    int m_signal_value;

#else
    HANDLE m_ra_ready_Sem;

    HANDLE m_lc_ready_Sem;
    HANDLE m_lc_lock;
    int m_lc_ready;
    int m_lc_status[MAX_CLIENT_NUM];

    HANDLE m_ref_ready_Sem;
    HANDLE m_ref_lock;
    int m_ref_ready;
    int m_ref_signal;

    HANDLE m_signal_lock;
    HANDLE m_signal_Sem;
    int m_signal_value;
#endif


/* constructor and destructor */
public:
    CClientPrimMgrThread(CSocketClient *socket);
	~CClientPrimMgrThread();

/* operations */
public:
    int GetLoadID();

    int GetStatus();

    void ResetRaReady();
    void WaitForRaReady();
    void SetRaReady();

    void ResetLcReady();
    BOOL WaitForLcReady(int *lc_status, int timeout=10000); /* timeout is in milliseconds */
    void AddLcReady(int lc_id, int lc_status);

    void SetRefClientNum(int ref_client_num);
    void ResetRefReady();
    int TimeWaitForRefReadySignal(int millisecond); /* return value: 0:timeout 1:ready 2:client-error 3,4:pause,stop-command */
    void AddRefReady();
    void SetRefSignal(int sig_code);                /* sig_code=2:client-error 3,4:pause,stop-command */

    void ResetSignal();
    int TimeWaitForSignal(int millisecond);  /* return value: 0:timeout >0:signal code */
    void SetSignal(int sig_code);

    void UpdateLoadInfo(CNet_StartLoadOnPrim *net_startload);
    void UpdateFileSysInfo(int client_id, int fs_index, CNet_FsSysInfo *net_fsinfo);
    void UpdateRefFileSysInfo(int fs_index, CNet_FsInfo *net_fsinfo);

    void ShuffleAllFileSystems();

    BOOL RecvFileSysInfoFromRM(CSocketClient *socket);
    BOOL RecvAckFromClient(CSocketClient *socket);
    BOOL RecvFileSysInfoFromClient(CSocketClient *socket);
    BOOL RecvRefFileSysInfoFromClient(CSocketClient *socket);

    BOOL SendLoadRunResponseToRM(CSocketClient *socket, BOOL succ, char *error_msg);
    BOOL SendLoadUpdateToRM(int status, int used_time, char *error_msg=NULL);

    BOOL SendStartLoadOnClients(CSocketClient *sock_to_clients, char *adv_buffer, int adv_size, char *error_msg);
    BOOL SendFSBaseInfoToClients(CSocketClient *sock_to_clients, char *error_msg);
    BOOL SendGoAheadToClients(CSocketClient *sock_to_clients, char *error_msg);
    BOOL SendFileSysInfoToClients(CSocketClient *sock_to_clients, char *error_msg);

    BOOL SendBreakMessageToClients(int status, char *error_msg);    /* with new connection */
    BOOL SendGoAheadToClients(char *error_msg);                     /* with new connection */
    BOOL SendFileSysInfoToClients(char *error_msg);                 /* with new connection */
    void SendQuitMessageClients();                                  /* with new connection */

    void ResetFileSysTotalNum();
    BOOL CheckForRefresh(int index);
    int RefeshFileSystems(int ref_fs_num, int ref_fs_index[]);      /* return value: 0:Error, 1:All finished 2:paused 3:stopped */

    void run();
    void run_once();

/* for meta session */
private:
    char m_ConfigFile[MAX_PATHLEN];
    CConfigFile m_config;

public:
    void SetConfigFile(char *config_file);

    BOOL ReadLoadInfo(char *error_msg);
    BOOL ReadAdvSetting(CNet_LoadAdvance *net_loadadv, char *error_msg);
    BOOL ReadFileSysInfo(BOOL succ[]);
    BOOL WriteFileSysInfo(char *error_msg);
};

#endif


