
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: FSInfoFile.cpp
**
** DESCRIPTION: This class mainly handle reading and writing file system
**              information file.
**              
**
** AUTHOR: Yubing Wang       Mar-01-2002
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
** FUNCTION LIST:
**     BOOL CFSInfoFile::ReadFileSysInfo(int id, CNet_FsInfo *net_fsinfo)
**     BOOL CFSInfoFile::WriteFileSysInfo(int id, CNet_FsInfo *net_fsinfo)
**
**
****************************************************************************/

#include "FSInfoFile.h"

BOOL CFSInfoFile::ReadFileSysInfo(char *dir, int id, CNet_FsInfo *net_fsinfo)
{
    char name[MAX_PATHLEN];
    if (dir[0] == '\0')     /* run at current dir */
        sprintf(name, "fs%04d.dat", id);
    else                    /* use given dir */
        sprintf(name, "%s%cfs%04d.dat", dir, _CHR_PATHSEP, id);

    FILE *fp = fopen(name, "r");
    if (fp == NULL)
        return FALSE;

    if (fscanf(fp, "%d\n", &net_fsinfo->TotalSize) != 1 || net_fsinfo->TotalSize < 0)
    {
        fclose(fp);
        return FALSE;
    }
    if (fscanf(fp, "%d\n", &net_fsinfo->TotalFNodeNum) != 1 || net_fsinfo->TotalFNodeNum < 0)
    {
        fclose(fp);
        return FALSE;
    }
//    if (fscanf(fp, "%d\n", &net_fsinfo->IODist) != 1 || net_fsinfo->IODist < 0)
//    {
//        fclose(fp);
//        return FALSE;
//    }
    if (fscanf(fp, "%d\n", &net_fsinfo->SubDirNum) != 1 || net_fsinfo->SubDirNum < 0)
    {
        fclose(fp);
        return FALSE;
    }
    if (fscanf(fp, "%d\n", &net_fsinfo->TotalIONum) != 1 || net_fsinfo->TotalIONum < 0)
    {
        fclose(fp);
        return FALSE;
    }
    if (fscanf(fp, "%d\n", &net_fsinfo->TotalFileNum) != 1 || net_fsinfo->TotalFileNum < 0)
    {
        fclose(fp);
        return FALSE;
    }
    if (fscanf(fp, "%d\n", &net_fsinfo->TotalDirNum) != 1 || net_fsinfo->TotalDirNum < 0)
    {
        fclose(fp);
        return FALSE;
    }
    if (fscanf(fp, "%d\n", &net_fsinfo->TotalSymlinkNum) != 1 || net_fsinfo->TotalSymlinkNum < 0)
    {
        fclose(fp);
        return FALSE;
    }

    int i, j, k, subdirnum, dirnum, symlinknum;
    int int_data;

    subdirnum = net_fsinfo->SubDirNum;

    net_fsinfo->IONum = new int [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        if (fscanf(fp, "%d\n", &net_fsinfo->IONum[i]) != 1 || net_fsinfo->IONum[i] < 0 || net_fsinfo->IONum[i] > net_fsinfo->TotalIONum)
        {
            delete [] net_fsinfo->IONum;
            fclose(fp);
            return FALSE;
        }
    }

    net_fsinfo->IOSize = new int * [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        net_fsinfo->IOSize[i] = new int [net_fsinfo->IONum[i]];
        for (j=0; j<net_fsinfo->IONum[i]; j++)
        {
            if (fscanf(fp, "%d\n", &net_fsinfo->IOSize[i][j]) != 1 || net_fsinfo->IOSize[i][j] < 0)
            {
                for (k=0; k<=i; k++)
                    delete [] net_fsinfo->IOSize[k];
                delete [] net_fsinfo->IOSize;
                delete [] net_fsinfo->IONum;
                fclose(fp);
                return FALSE;
            }
        }
    }

    net_fsinfo->FileNum = new int [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        if (fscanf(fp, "%d\n", &net_fsinfo->FileNum[i]) != 1 || net_fsinfo->FileNum[i] < 0 || net_fsinfo->FileNum[i] > net_fsinfo->TotalFileNum)
        {
            delete [] net_fsinfo->FileNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->IOSize[k];
            delete [] net_fsinfo->IOSize;
            delete [] net_fsinfo->IONum;
            fclose(fp);
            return FALSE;
        }
    }

    net_fsinfo->FileMode = new ushort * [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        net_fsinfo->FileMode[i] = new ushort [net_fsinfo->FileNum[i]];
        for (j=0; j<net_fsinfo->FileNum[i]; j++)
        {
            if (fscanf(fp, "%d\n", &int_data) != 1)
            {
                for (k=0; k<=i; k++)
                    delete [] net_fsinfo->FileMode[k];
                delete [] net_fsinfo->FileMode;
                delete [] net_fsinfo->FileNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->IOSize[k];
                delete [] net_fsinfo->IOSize;
                delete [] net_fsinfo->IONum;
                fclose(fp);
                return FALSE;
            }
            net_fsinfo->FileMode[i][j] = (ushort) int_data;
        }
    }

    net_fsinfo->DirNum = new int [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        if (fscanf(fp, "%d\n", &net_fsinfo->DirNum[i]) != 1 || net_fsinfo->DirNum[i] < 0 || net_fsinfo->DirNum[i] > net_fsinfo->TotalDirNum)
        {
            delete [] net_fsinfo->DirNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->FileMode[k];
            delete [] net_fsinfo->FileMode;
            delete [] net_fsinfo->FileNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->IOSize[k];
            delete [] net_fsinfo->IOSize;
            delete [] net_fsinfo->IONum;
            fclose(fp);
            return FALSE;
        }
    }

    net_fsinfo->DirMode = new uchar * [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        dirnum = SIZE_BIT_TO_BYTE(net_fsinfo->DirNum[i]);
        net_fsinfo->DirMode[i] = new uchar [dirnum];
        for (j=0; j<dirnum; j++)
        {
            if (fscanf(fp, "%d\n", &int_data) != 1)
            {
                for (k=0; k<=i; k++)
                    delete [] net_fsinfo->DirMode[k];
                delete [] net_fsinfo->DirMode;
                delete [] net_fsinfo->DirNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->FileMode[k];
                delete [] net_fsinfo->FileMode;
                delete [] net_fsinfo->FileNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->IOSize[k];
                delete [] net_fsinfo->IOSize;
                delete [] net_fsinfo->IONum;
                fclose(fp);
                return FALSE;
            }
            net_fsinfo->DirMode[i][j] = (uchar) int_data;
        }
    }

    net_fsinfo->SymlinkNum = new int [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        if (fscanf(fp, "%d\n", &net_fsinfo->SymlinkNum[i]) != 1 || net_fsinfo->SymlinkNum[i] < 0 || net_fsinfo->SymlinkNum[i] > net_fsinfo->TotalSymlinkNum)
        {
            delete [] net_fsinfo->SymlinkNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->DirMode[k];
            delete [] net_fsinfo->DirMode;
            delete [] net_fsinfo->DirNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->FileMode[k];
            delete [] net_fsinfo->FileMode;
            delete [] net_fsinfo->FileNum;
            for (k=0; k<subdirnum; k++)
                delete [] net_fsinfo->IOSize[k];
            delete [] net_fsinfo->IOSize;
            delete [] net_fsinfo->IONum;
            fclose(fp);
            return FALSE;
        }
    }

    net_fsinfo->SymlinkMode = new uchar * [subdirnum];
    for (i=0; i<subdirnum; i++)
    {
        symlinknum = SIZE_BIT_TO_BYTE(net_fsinfo->SymlinkNum[i]);
        net_fsinfo->SymlinkMode[i] = new uchar [symlinknum];
        for (j=0; j<symlinknum; j++)
        {
            if (fscanf(fp, "%d\n", &int_data) != 1)
            {
                for (k=0; k<=i; k++)
                    delete [] net_fsinfo->SymlinkMode[k];
                delete [] net_fsinfo->SymlinkMode;
                delete [] net_fsinfo->SymlinkNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->DirMode[k];
                delete [] net_fsinfo->DirMode;
                delete [] net_fsinfo->DirNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->FileMode[k];
                delete [] net_fsinfo->FileMode;
                delete [] net_fsinfo->FileNum;
                for (k=0; k<subdirnum; k++)
                    delete [] net_fsinfo->IOSize[k];
                delete [] net_fsinfo->IOSize;
                delete [] net_fsinfo->IONum;
                fclose(fp);
                return FALSE;
            }
            net_fsinfo->SymlinkMode[i][j] = (uchar) int_data;
        }
    }

    fclose(fp);

    return TRUE;
}

BOOL CFSInfoFile::WriteFileSysInfo(char *dir, int id, CNet_FsInfo *net_fsinfo)
{
    char name[MAX_PATHLEN];
    if (dir[0] == '\0')     /* run at current dir */
        sprintf(name, "fs%04d.dat", id);
    else                    /* use given dir */
        sprintf(name, "%s%cfs%04d.dat", dir, _CHR_PATHSEP, id);
    FILE *fp = fopen(name, "w");
    if (fp == NULL)
        return FALSE;

    fprintf(fp, "%d\n", net_fsinfo->TotalSize);
    fprintf(fp, "%d\n", net_fsinfo->TotalFNodeNum);
//    fprintf(fp, "%d\n", net_fsinfo->IODist);
    fprintf(fp, "%d\n", net_fsinfo->SubDirNum);
    fprintf(fp, "%d\n", net_fsinfo->TotalIONum);
    fprintf(fp, "%d\n", net_fsinfo->TotalFileNum);
    fprintf(fp, "%d\n", net_fsinfo->TotalDirNum);
    fprintf(fp, "%d\n", net_fsinfo->TotalSymlinkNum);


    int i, j, subdirnum, dirnum, symlinknum;
    subdirnum = net_fsinfo->SubDirNum;
    for (i=0; i<subdirnum; i++)
        fprintf(fp, "%d\n", net_fsinfo->IONum[i]);
    for (i=0; i<subdirnum; i++)
    {
        for (j=0; j<net_fsinfo->IONum[i]; j++)
            fprintf(fp, "%d\n", net_fsinfo->IOSize[i][j]);
    }
    for (i=0; i<subdirnum; i++)
        fprintf(fp, "%d\n", net_fsinfo->FileNum[i]);
    for (i=0; i<subdirnum; i++)
    {
        for (j=0; j<net_fsinfo->FileNum[i]; j++)
            fprintf(fp, "%d\n", (int)net_fsinfo->FileMode[i][j]);
    }
    for (i=0; i<subdirnum; i++)
        fprintf(fp, "%d\n", net_fsinfo->DirNum[i]);
    for (i=0; i<subdirnum; i++)
    {
        dirnum = SIZE_BIT_TO_BYTE(net_fsinfo->DirNum[i]);
        for (j=0; j<dirnum; j++)
            fprintf(fp, "%d\n", (int)net_fsinfo->DirMode[i][j]);
    }

    for (i=0; i<subdirnum; i++)
        fprintf(fp, "%d\n", net_fsinfo->SymlinkNum[i]);
    for (i=0; i<subdirnum; i++)
    {
        symlinknum = SIZE_BIT_TO_BYTE(net_fsinfo->SymlinkNum[i]);
        for (j=0; j<symlinknum; j++)
            fprintf(fp, "%d\n", (int)net_fsinfo->SymlinkMode[i][j]);
    }

    fclose(fp);

    return TRUE;
}

