
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: ClientOperation.cpp
**
** DESCRIPTION: These classes provide the functions to execute an operation.
**              
**
** AUTHOR: Yubing Wang       Mar-30-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
** FUNCTION LIST:
**     CClientOperation::CClientOperation()
**     CClientOperation::~CClientOperation()
**
**     CClientOprtRead::CClientOprtRead()
**     CClientOprtRead::~CClientOprtRead()
**
**     CClientOprtWrite::CClientOprtWrite()
**     CClientOprtWrite::~CClientOprtWrite()
**
**     CClientOprtGetAttr::CClientOprtGetAttr()
**     CClientOprtGetAttr::~CClientOprtGetAttr()
**
**
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Common.h"
#include "WinClientOperation.h"
#include "ClientLoadMgrThread.h"


static CClientOprtRead              op0;
static CClientOprtOverWrite         op1;
static CClientOprtAppend            op2;
static CClientOprtTruncate          op3;
static CClientOprtGetAttr           op4;
static CClientOprtSetAttr           op5;
static CClientOprtCreatNULL         op6;
static CClientOprtMkdir             op7;
static CClientOprtRmdir             op8;
static CClientOprtReadDir           op9;
static CClientOprtRename            op10;
static CClientOprtDelete            op11;
static CClientOprtGetDiskSpace      op12;

Op_Map Ops_Pcnt[MAX_OPNUM+1] = {

/* Op Class         mix   call   req  req  req   op class
    name            pcnt  targ   pcnt cnt  targ    ptr       */

 { "Read",           20,   0,    0,    0,   0,    &op0  },
 { "OverWrite",      20,   0,    0,    0,   0,    &op1  },
 { "Append",         7,    0,    0,    0,   0,    &op2  },
 { "Truncate",       4,    0,    0,    0,   0,    &op3  },
 { "GetAttr",        10,   0,    0,    0,   0,    &op4  },
 { "SetAttr",        10,   0,    0,    0,   0,    &op5  },
 { "CreatNULL",      5,    0,    0,    0,   0,    &op6  },
 { "Mkdir",          5,    0,    0,    0,   0,    &op7  },
 { "Rmdir",          5,    0,    0,    0,   0,    &op8  },
 { "ReadDir",        4,    0,    0,    0,   0,    &op9  },
 { "Rename",         4,    0,    0,    0,   0,    &op10 },
 { "Delete",         5,    0,    0,    0,   0,    &op11 },
 { "GetDiskSpace",   1,    0,    0,    0,   0,    &op12 },
 { "TOTAL",                     100,  0,    0,    0,   0,    NULL  }
};

/************************************************************/
/*           CClientOperation : the base class              */
/************************************************************/
CClientOperation::CClientOperation()
{
}

CClientOperation::CClientOperation(CClientLoadMgrThread *host)
{
    load_mgr_ptr = host;
}

CClientOperation::~CClientOperation()
{
}

CClientOperation *CClientOperation::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOperation(host);
}


BOOL CClientOperation::DoOP()
{
    return TRUE;
}

/************************************************************/
/*                    CClientOprtRead                       */
/************************************************************/
CClientOprtRead::CClientOprtRead() : CClientOperation()
{
}

CClientOprtRead::CClientOprtRead(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRead::~CClientOprtRead()
{
}

CClientOperation * CClientOprtRead::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRead(host);
}


BOOL CClientOprtRead::DoOP()
{
    int   syscall_ret;               /* result from system call */
    int   cur_cnt;                   /* packet loop variable */
    int   max_cnt;                   /* packet number */
    int   offset;                    /* offset position reading from */
    int   size;                      /* data size each reading */
    int   nRead;                     // data size read for each reading
    int   frag_size;                 /* data size of the fragment */
    int   total_size;                /* data size read */

    /* get transfer size for read operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(1);

    /* Don't allow a read of less than one block size */
    //Bytes_per_block = kb_per_block * 1024;
    if (xfer_size < Bytes_per_block)
        xfer_size = Bytes_per_block;

    /* pick a file for read operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do read\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);

    /* randomly choose an offset that is a multiple of the block size
     * and constrained by making the transfer fit within the file */
    offset = 0;
    if (file_size > xfer_size)
        offset = Bytes_per_block * (GetRandom() % (((file_size - xfer_size) / Bytes_per_block) + 1));

    _ASSERT(file_size >= offset + xfer_size);

//    printf("Read file (%s, size=%06x) %06x bytes (from %06x to %06x)\n", file_name, file_size, xfer_size, offset, offset+xfer_size);

    /* open the file for reading */
    file_handle = CreateFile(file_name,         // open filename 
                      GENERIC_READ,             // open for reading 
                      0,                        // don't share for reading 
                      NULL,                     // no security 
                      OPEN_EXISTING,            // existing file only 
                      FILE_ATTRIBUTE_NORMAL,    // normal file
                      NULL);                    // no attr. template
    if (file_handle == INVALID_HANDLE_VALUE)
        syscall_ret = -1;
    else
    {
        SetFilePointer(file_handle,             // handle to file
                       (LONG) offset,           // bytes to move pointer
                       (PLONG) NULL,            // bytes to move pointer
                       FILE_BEGIN);             // starting point
        syscall_ret = 1;
    }

    if (syscall_ret < 0)
    {
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }

    /* start to read. read the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;
    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;
    
    if (frag_size > 0)
        max_cnt ++;

    total_size = 0;

//XXX
CSysTime a;
    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should read fragment */
        if (total_size > xfer_size)
            size = frag_size;

        syscall_ret = ReadFile(file_handle,         // handle to file
                               (LPVOID)buf,         // data buffer
                               (DWORD)size,         // number of bytes to read
                               (LPDWORD)&nRead,     // number of bytes read
                               (LPOVERLAPPED)NULL); // nooverlapped buffer;

        if (! syscall_ret || nRead < size)
        {
            fprintf(stderr, "read call error on file %s\n", file_name);
        }
    } /* for reading max_cnt packets */


    /* close the file descriptor */
    CloseHandle(file_handle);
//XXX
CSysTime b;
b-=a;
if (b.m_sec > 0)
{
printf("read file %s (%d bytes): %d sec\n", file_name, xfer_size, b.m_sec);
}

    /* set back the file size to release this file */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);

    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}


/************************************************************/
/*                    CClientOprtOverWrite                  */
/************************************************************/
unsigned int CClientOprtOverWrite::m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

CClientOprtOverWrite::CClientOprtOverWrite() : CClientOperation()
{
}

CClientOprtOverWrite::CClientOprtOverWrite(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtOverWrite::~CClientOprtOverWrite()
{
}

void CClientOprtOverWrite::InitWriteBuffer()
{
    unsigned int *bp;
    unsigned int *be = m_WriteBuf + (sizeof(m_WriteBuf) / sizeof(unsigned int));

    for (bp = m_WriteBuf; bp < be; bp++)
        *bp = (unsigned int)0xdeadbeef;
}

CClientOperation * CClientOprtOverWrite::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtOverWrite(host);
}


BOOL CClientOprtOverWrite::DoOP()
{
    int             syscall_ret;               /* result from system call */
    int             cur_cnt;                   /* packet loop variable */
    int             max_cnt;               /* packet number */
    int             offset;                    /* offset position writing from */
    int             size;                      /* data size each writing */
    int             nWritten;                  // data size written for each writing
    int             frag_size;                 /* data size of the fragment */
    int             total_size;                /* data size written */

    /* get transfer size for overwrite operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* Don't allow a read of less than one block size */
    if (xfer_size < Bytes_per_block)
        xfer_size = Bytes_per_block;

    /* pick a file for overwrite operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do overwrite\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);

    /* if the transfer data size large than 128 blocks, then set the flush flag */
    BOOL FlushFlag = FALSE;
    if (xfer_size > Bytes_per_block * 128)
        FlushFlag = TRUE;

    /* randomly choose an offset that is a multiple of the block size
     * and constrained by making the transfer fit within the file */
    offset = 0;
    if (file_size > xfer_size)
        offset = Bytes_per_block * (GetRandom() % (((file_size - xfer_size) / Bytes_per_block) + 1));

    _ASSERT(file_size >= offset + xfer_size);

//    printf("Write file (%s, size=%06x) %06x bytes (from %06x to %06x)\n", file_name, file_size, xfer_size, offset, offset+xfer_size);
    
    /* open the file for writing */
    file_handle = CreateFile(file_name,         // open filename 
                       GENERIC_WRITE,           // open for writing
                       0,                       // don't share for reading 
                       NULL,                    // no security 
                       OPEN_EXISTING,           // existing file only 
                       FILE_ATTRIBUTE_NORMAL,   // normal file
                       NULL);                   // no attr. template
    if (file_handle == INVALID_HANDLE_VALUE)
        syscall_ret = -1;
    else
    {
        
        DWORD moveRes = SetFilePointer(file_handle,             // handle to file
                           (LONG) offset,       // bytes to move pointer
                           (PLONG) NULL,        // bytes to move pointer
                           FILE_BEGIN);         // starting point at the offset
        if (moveRes == INVALID_FILE_SIZE)
            syscall_ret = -1;
        else
            syscall_ret = 1;
    }

    if (syscall_ret < 0)
    {
        fprintf(stderr, "overwrite call (open or setfilepointer) error on file %s\n",file_name);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }    
 
    /* start to write. write the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;

    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;

    if (frag_size > 0)
        max_cnt ++;
    
    total_size = 0;
    
//XXX
CSysTime a;
    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should write fragment */
        if (total_size > xfer_size)
            size = frag_size;
 
        syscall_ret = WriteFile(file_handle,            // handle to file
                                (LPVOID)m_WriteBuf,     // data buffer
                                (DWORD) size,           // number of bytes to write
                                (LPDWORD) &nWritten,    // number of bytes written
                                (LPOVERLAPPED) NULL);   // overlapped buffer
    
        if (! syscall_ret || nWritten < size)
        {
            fprintf(stderr, "overwrite call error on file %s: (block %d)\n", file_name, cur_cnt);
        } 
    } /* for writing max_cnt packets */
//XXX
CSysTime b;

    if (FlushFlag)
        FlushFileBuffers(file_handle);

    CloseHandle(file_handle);

    /* set back the file size to release this file */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);

//XXX
b-=a;
if (b.m_sec > 0)
{
printf("write file %s (%d bytes): %d sec\n", file_name, xfer_size, b.m_sec);
}

    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}

/************************************************************/
/*                   CClientOprtAppend                      */
/************************************************************/
unsigned int CClientOprtAppend::m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

CClientOprtAppend::CClientOprtAppend() : CClientOperation()
{
}

CClientOprtAppend::CClientOprtAppend(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtAppend::~CClientOprtAppend()
{
}

CClientOperation * CClientOprtAppend::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtAppend(host);
}


void CClientOprtAppend::InitWriteBuffer()
{
    unsigned int *bp;
    unsigned int *be  = m_WriteBuf + (sizeof(m_WriteBuf) / sizeof(unsigned int));

    for (bp = m_WriteBuf; bp < be; bp++)
        *bp = (unsigned int)0xdeadbeef;
}

BOOL CClientOprtAppend::DoOP()
{
    int             syscall_ret;        /* result from system call */
    int             cur_cnt;            /* packet loop variable */
    int             max_cnt;            /* packet number */
    int             size;               /* data size each writing */
    int             nWritten;           /* data size written for each writing */
    int             frag_size;          /* data size of the fragment */
    int             total_size;         /* data size written */

    /* get transfer size for append operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* Don't allow a read of less than one block size */
    if (xfer_size < Bytes_per_block)
	    xfer_size = Bytes_per_block;

    /* pick a file for append operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(0, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do append\n");
        return FALSE;
    }

    /* if the transfer data size large than 128 blocks, then set the flush flag */
    BOOL FlushFlag = FALSE;
    if (xfer_size > Bytes_per_block * 128)
        FlushFlag = TRUE;

    /* open the file for appending */
    file_handle = CreateFile(file_name,         // open filename 
                       GENERIC_WRITE,           // open for writing
                       0,                       // don't share for reading 
                       NULL,                    // no security 
                       OPEN_EXISTING,           // existing file only 
                       FILE_ATTRIBUTE_NORMAL,   // normal file
                       NULL);                   // no attr. template
    
    if (file_handle == INVALID_HANDLE_VALUE)
        syscall_ret = -1;
    else
    {
        SetFilePointer(file_handle,             // handle to file
                       0L,                      // bytes to move pointer
                       (PLONG) NULL,            // bytes to move pointer
                       FILE_END);               // starting point at the end of file
        syscall_ret = 1;
    }
        
    if (syscall_ret < 0)
    {
        fprintf(stderr, "append call error on file %s\n", file_name);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }    

    /* start to write. write the whole buffers and fragment */
    size = Bytes_per_block;
    max_cnt = xfer_size / Bytes_per_block;

    /* fragment size : 1KB ~ (Kb_per_block -1) KB fragment */
    frag_size = xfer_size % Bytes_per_block;

    if (frag_size > 0)
        max_cnt ++;
    
    total_size = 0;
    
    /* make the call(s) now */
    for (cur_cnt = 0; cur_cnt < max_cnt; cur_cnt++)
    {
        total_size += size;

        /* decide if it should write fragment */
        if (total_size > xfer_size)
            size = frag_size;

        syscall_ret = WriteFile(file_handle,            // handle to file
                                (LPVOID)m_WriteBuf,     // data buffer
                                (DWORD) size,           // number of bytes to write
                                (LPDWORD) &nWritten,    // number of bytes written
                                (LPOVERLAPPED) NULL);   // overlapped buffer

        if (! syscall_ret || nWritten < size)
        {
            fprintf(stderr, "%s: append call error on file %d\n", file_name);
        } 
    } /* for writing max_cnt packets */

    if (FlushFlag)
        FlushFileBuffers(file_handle);

    CloseHandle(file_handle);

    /* set the new file size */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size + xfer_size);

    load_mgr_ptr->AddOps(max_cnt);

    return TRUE;
}

/************************************************************/
/*                   CClientOprtTruncate                    */
/************************************************************/
CClientOprtTruncate::CClientOprtTruncate() : CClientOperation()
{
}

CClientOprtTruncate::CClientOprtTruncate(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtTruncate::~CClientOprtTruncate()
{
}

CClientOperation * CClientOprtTruncate::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtTruncate(host);
}


BOOL CClientOprtTruncate::DoOP()
{
    BOOL            ret;
   
    /* get truncating size for truncate operation */
    xfer_size = load_mgr_ptr->GetFileSysManager()->GetIOOpSize(2);

    /* pick a file for truncate operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetIOFileInfo(xfer_size, file_name, fs, fd, file_size))
    {
        fprintf(stderr, "Can't find a file to do truncate\n");
        return FALSE;
    }

    /* assure file size is larger than the transfer size */
    _ASSERT(file_size >= xfer_size);
   
    /* make the call */
    file_handle = CreateFile(file_name,        // open filename
                    GENERIC_WRITE,             // open for writing 
                    0,                         // don't share for reading 
                    NULL,                      // no security 
                    OPEN_EXISTING,             // open an existing file
                    FILE_ATTRIBUTE_NORMAL,     // normal file 
                    NULL);                     // no attr. template

    if (file_handle == INVALID_HANDLE_VALUE)
    {
        fprintf(stderr, "truncate call error on file %s: can't open file\n", file_name);
        load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size);
        return FALSE;
    }    

    if (SetFilePointer(file_handle, (LONG)(file_size - xfer_size), NULL, FILE_BEGIN) == INVALID_FILE_SIZE)
    {
        fprintf(stderr, "truncate call error on file %s: can't SetFilePointer\n", file_name);
        ret = FALSE;
    } 
    else if (! SetEndOfFile(file_handle))
    {
        fprintf(stderr, "truncate call error on file %s: can't SetEndOfFile\n", file_name);
        ret = FALSE;
    } 
    else
        ret = TRUE;
        
    CloseHandle(file_handle);

    /* set the new file size */
    load_mgr_ptr->GetFileSysManager()->SetIOFileInfo(fs, fd, file_size - xfer_size);
    
    load_mgr_ptr->AddOps(1);

    return ret;
}


/************************************************************/
/*                   CClientOprtGetAttr                     */
/************************************************************/
CClientOprtGetAttr::CClientOprtGetAttr() : CClientOperation()
{
    
}

CClientOprtGetAttr::CClientOprtGetAttr(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtGetAttr::~CClientOprtGetAttr()
{
}

CClientOperation * CClientOprtGetAttr::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtGetAttr(host);
}

BOOL CClientOprtGetAttr::DoOP()
{
    DWORD               fileattr;       // the attribute of this file
    BOOL                ret;
    
    
    /* pick a file for getattr operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do getattr\n");
        return FALSE;
    }

    /* make the call */
    fileattr = GetFileAttributes(file_name);

    if (fileattr != 0xFFFFFFFF)
    {
        ret = TRUE;
    }
    else
    {
        fprintf(stderr, "getattr error on file %s\n", file_name);
        ret = FALSE;
    } 
    
    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}
/************************************************************/
/*                   CClientOprtSetAttr                     */
/************************************************************/
CClientOprtSetAttr::CClientOprtSetAttr() : CClientOperation()
{
}

CClientOprtSetAttr::CClientOprtSetAttr(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtSetAttr::~CClientOprtSetAttr()
{
}

CClientOperation * CClientOprtSetAttr::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtSetAttr(host);
}

BOOL CClientOprtSetAttr::DoOP()
{
    BOOL ret;       /* result from system call */

    /* pick a file for setattr operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do setattr\n");
        return FALSE;
    }

    /* make the call */
    ret = SetFileAttributes(file_name, FILE_ATTRIBUTE_ARCHIVE | FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_READONLY);
    if (ret)
        ret = SetFileAttributes(file_name, FILE_ATTRIBUTE_NORMAL);

    if (! ret)
    {
        fprintf(stderr, "setattr call error on file %s\n", file_name);
    } 

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtCreatNULL                   */
/************************************************************/
CClientOprtCreatNULL::CClientOprtCreatNULL() : CClientOperation()
{
}

CClientOprtCreatNULL::CClientOprtCreatNULL(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtCreatNULL::~CClientOprtCreatNULL()
{
}

CClientOperation * CClientOprtCreatNULL::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtCreatNULL(host);
}

BOOL CClientOprtCreatNULL::DoOP()
{
    BOOL ret;       /* result from system call */

    /* pick a file for create operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(2, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do create\n");
        return FALSE;
    }

    /* make the call */
    file_handle = CreateFile(file_name,             // open filename
                             GENERIC_WRITE,         // open for writing
                             0,                     // don't share for writing
                             NULL,                  // no security
                             CREATE_ALWAYS,         // creating a new file
                             FILE_ATTRIBUTE_NORMAL, // normal file
                             NULL);                 // no attr. template

    if (file_handle == INVALID_HANDLE_VALUE)
    {
        fprintf(stderr, "creat call error on file %s\n", file_name);
        ret = FALSE;
    }
    else 
    {
        CloseHandle(file_handle);
        ret = TRUE;      
    } 

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(2, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                    CClientOprtMkdir                      */
/************************************************************/
CClientOprtMkdir::CClientOprtMkdir() : CClientOperation()
{
}

CClientOprtMkdir::CClientOprtMkdir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtMkdir::~CClientOprtMkdir()
{
}

CClientOperation * CClientOprtMkdir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtMkdir(host);
}

BOOL CClientOprtMkdir::DoOP()
{
    BOOL ret;       /* result from system call */

    /* pick a directory for mkdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(2, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do mkdir\n");
        return FALSE;
    }

    /* make the call */
    ret = CreateDirectory(dir_name, NULL);;

    if (! ret) 
    {
        fprintf(stderr, "mkdir call error on dir %s\n", dir_name);
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(2, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                    CClientOprtRmdir                      */
/************************************************************/
CClientOprtRmdir::CClientOprtRmdir() : CClientOperation()
{
}

CClientOprtRmdir::CClientOprtRmdir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRmdir::~CClientOprtRmdir()
{
}


CClientOperation * CClientOprtRmdir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRmdir(host);
}

BOOL CClientOprtRmdir::DoOP()
{
    BOOL ret;       /* result from system call */

    /* pick a directory for rmdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(1, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do rmdir\n");
        return FALSE;
    }

    /* make the call */
    ret = RemoveDirectory(dir_name);

    if (! ret) 
    {
        fprintf(stderr, "Rmdir call error on dir %s\n", dir_name);
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(1, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtReadDir                     */
/************************************************************/
CClientOprtReadDir::CClientOprtReadDir() : CClientOperation()
{
}

CClientOprtReadDir::CClientOprtReadDir(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtReadDir::~CClientOprtReadDir()
{
}

CClientOperation * CClientOprtReadDir::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtReadDir(host);
}

BOOL CClientOprtReadDir::DoOP()
{
    HANDLE hSearch;             /* handle of directory search */
    WIN32_FIND_DATA FileData;
    char filepath[MAX_PATHLEN];
    char *chptr;
    BOOL ret;                   /* result from system call */

    /* pick a directory for readdir operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetDirInfo(0, dir_name, fs, fd))
    {
        fprintf(stderr, "Can't find a dir to do readdir\n");
        return FALSE;
    }

    lstrcpy(filepath, dir_name);
    chptr = strrchr(filepath, '\\');
    if (chptr != NULL)
        lstrcpy(chptr+1, "*.*");
    else
        lstrcat(filepath, "\\*.*");

    /* make the call */
    hSearch = FindFirstFile(filepath, &FileData);

    if (hSearch == INVALID_HANDLE_VALUE)
    {
        fprintf(stderr, "Readdir call error on dir %s\n", dir_name);
        ret = FALSE;
    }
    else 
    {
        FindClose(hSearch);
        ret = TRUE;
    }

    /* set back the dir mode to release this directory */
    load_mgr_ptr->GetFileSysManager()->SetDirInfo(0, fs, fd);

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtRename                      */
/************************************************************/
CClientOprtRename::CClientOprtRename() : CClientOperation()
{
}

CClientOprtRename::CClientOprtRename(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtRename::~CClientOprtRename()
{
}


CClientOperation * CClientOprtRename::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtRename(host);
}

BOOL CClientOprtRename::DoOP()
{
    BOOL ret;               /* result from system call */
    int fs2, fd2, mode2;    /* for second file */

    /* pick two files for rename operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(1, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find an existing file to do rename\n");
        return FALSE;
    }
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(2, target_file_name, fs2, fd2, mode2))
    {
        /* don't delete this file, so release it using access option */
        load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);
        fprintf(stderr, "Can't find a new file to do rename\n");
        return FALSE;
    }
    mode2 = mode;

    /* make the call */
    ret = SetFileAttributes(file_name, FILE_ATTRIBUTE_NORMAL);
    if (ret)
        ret = MoveFile(file_name, target_file_name);

    if (! ret) 
    {
        fprintf(stderr, "rename call error on file %s & %s\n", file_name, target_file_name);
    }

    /* set back the two files' mode to release these files */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(1, fs, fd, mode);
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(2, fs2, fd2, mode2);    

    load_mgr_ptr->AddOps(1);

    return ret;
}

/************************************************************/
/*                   CClientOprtDelete                      */
/************************************************************/
CClientOprtDelete::CClientOprtDelete() : CClientOperation()
{
}

CClientOprtDelete::CClientOprtDelete(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtDelete::~CClientOprtDelete()
{
}


CClientOperation * CClientOprtDelete::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtDelete(host);
}

BOOL CClientOprtDelete::DoOP()
{
    BOOL ret;               /* result from system call */

    /* pick a file for delete operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(1, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do delete\n");
        return FALSE;
    }

    /* make the call */
    ret = SetFileAttributes(file_name, FILE_ATTRIBUTE_NORMAL);
    if (ret)
        ret = DeleteFile(file_name);

    if (! ret) 
    {
        fprintf(stderr, "delete call error on file %s\n", file_name);
    }

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(1, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}


/************************************************************/
/*                CClientOprtGetDiskSpace                   */
/************************************************************/

CClientOprtGetDiskSpace::CClientOprtGetDiskSpace() : CClientOperation()
{
}

CClientOprtGetDiskSpace::CClientOprtGetDiskSpace(CClientLoadMgrThread *host) : CClientOperation(host)
{
}

CClientOprtGetDiskSpace::~CClientOprtGetDiskSpace()
{
}


CClientOperation * CClientOprtGetDiskSpace::DupObj(CClientLoadMgrThread *host)
{
    return new CClientOprtGetDiskSpace(host);
}
   

BOOL CClientOprtGetDiskSpace::DoOP()
{
    char *chptr;                    /* root path name */
    DWORD SectorsPerCluster;        /* sectors per cluster */
    DWORD BytesPerSector;           /* bytes per sector */
    DWORD NumberOfFreeClusters;     /* free clusters */
    DWORD TotalNumberOfClusters;    /* total clusters */
    BOOL ret;                       /* result from system call */

    /* pick a file for getdiskspace operation */
    if (! load_mgr_ptr->GetFileSysManager()->GetNonIOFileInfo(0, file_name, fs, fd, mode))
    {
        fprintf(stderr, "Can't find a file to do GetDiskFreeSpace\n");
        return FALSE;
    }

    ret = TRUE;
    chptr = strchr(file_name, ':');
    if (chptr != NULL)
    {
        *(chptr+1) = '\\';
        *(chptr+2) = 0;
    }
    else if (strncmp(file_name, "\\\\", 2) == 0)
    {
        chptr = strchr(file_name+2, '\\');
        if (chptr != NULL)
        {
            chptr = strchr(chptr+1, '\\');
            if (chptr != NULL)
                *(chptr+1) = 0;
            else
                strcat(file_name, "\\");
        }
        else
            ret = FALSE;
    }
    else
        ret = FALSE;

    if (ret)
    {
        ret = GetDiskFreeSpace((LPCTSTR) file_name,         // root path
                               &SectorsPerCluster,          // sectors per cluster
                               &BytesPerSector,             // bytes per sector
                               &NumberOfFreeClusters,       // free clusters
                               &TotalNumberOfClusters);     // total clusters
    }

    if (! ret) 
    {
        fprintf(stderr, "GetDiskFreeSpace call error on file %s: %d\n", file_name, GetLastError());
    }

    /* set back the file mode to release this file */
    load_mgr_ptr->GetFileSysManager()->SetNonIOFileInfo(0, fs, fd, mode);

    load_mgr_ptr->AddOps(1);

    return ret;
}
