
/**************************************************************************
**
** Copyright (C) EMC Corporation 2001, All rights reserved.
**
** FILE NAME: WinClientOperation.h
**
** DESCRIPTION: This is header file to define the class:
**                  CClientOperation
**              and its subclasses:
**                  CClientOprtRead
**                  CClientOprtWrite
**                  CClientOprtGetAttr
**                    ... ...
**
**              These classes implememt the opreations for load generation.
**              
**
** AUTHOR: Yubing Wang       Mar-30-2001
**
** REVISION HISTORY:
** 
**  Who         Date       Description of Change
** ---------------------------------------------------------------------
**
** (This is only for the major code change that does not include bug fixes.
** Comments for bug fixes should be placed directly where the bugs are.)
**
****************************************************************************
**
**
**  CLASS LIST:
**      CClientOperation
**      CClientOprtRead
**      CClientOprtOverWrite
**      CClientOprtAppend
**      CClientOprtTruncate
**      CClientOprtGetAttr
**      CClientOprtSetAttr
**      CClientOprtCreatNULL
**      CClientOprtMkdir
**      CClientOprtRmdir
**      CClientOprtReadDir
**      CClientOprtRename
**      CClientOprtDelete
**      CClientOprtMemmap
**      CClientOprtGetDiskSpace
**
****************************************************************************/

#ifndef _CClientWinOperation_H_
#define _CClientWinOperation_H_

//#include <sys/types.h>
//#include <sys/stat.h>
//#include <sys/statvfs.h>
#include "Common.h"
#include "SysTime.h"

class CClientLoadMgrThread;

#define Bytes_per_block  8192
#define MAX_WRTBUFSIZE   32768

/************************************************************/
/*         CClientOperation : the base class             */
/************************************************************/
class CClientOperation
{
private:

protected:
    int     xfer_size;
    int     fs;
    int     fd;
    HANDLE  file_handle;
    int     mode;
    char    file_name[MAX_PATHLEN];
    char    target_file_name[MAX_PATHLEN];
    char    dir_name[MAX_PATHLEN];
    char    target_dir_name[MAX_PATHLEN];
    int     file_size;
    char    buf[MAX_BUFSIZE]; /* data buffer */
    int     kb_per_block;
    int     good_calls;
    int     bad_calls;
    CClientLoadMgrThread *load_mgr_ptr;

/* constructor and destructor */
public:
    CClientOperation();
    CClientOperation(CClientLoadMgrThread *host);
    ~CClientOperation();

/* operations */
public:
    virtual CClientOperation *DupObj(CClientLoadMgrThread *host);
    virtual BOOL DoOP();
    
};


/************************************************************/
/*                    CClientOprtRead                       */
/************************************************************/
class CClientOprtRead : public CClientOperation
{
private:
    
    /* constructor and destructor */
public:
    CClientOprtRead();
    CClientOprtRead(CClientLoadMgrThread *host);
    ~CClientOprtRead();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};


/************************************************************/
/*                    CClientOprtOverWrite                      */
/************************************************************/
class CClientOprtOverWrite : public CClientOperation
{
private:
    static unsigned int m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

/* constructor and destructor */
public:
    CClientOprtOverWrite();
    CClientOprtOverWrite(CClientLoadMgrThread *host);
    ~CClientOprtOverWrite();
    
/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);

    static void InitWriteBuffer();

    BOOL  DoOP();
};

/************************************************************/
/*                    CClientOprtAppend                      */
/************************************************************/
class CClientOprtAppend : public CClientOperation
{
private:
    static unsigned int m_WriteBuf[MAX_WRTBUFSIZE / sizeof(unsigned int)]; /* data buffer */

/* constructor and destructor */
public:
    CClientOprtAppend();
    CClientOprtAppend(CClientLoadMgrThread *host);
    ~CClientOprtAppend();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    static void InitWriteBuffer();
    BOOL  DoOP();
};

/************************************************************/
/*                    CClientOprtTruncate                      */
/************************************************************/
class CClientOprtTruncate : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtTruncate();
    CClientOprtTruncate(CClientLoadMgrThread *host);
    ~CClientOprtTruncate();

/* operations */
public:
   CClientOperation *DupObj(CClientLoadMgrThread *host);
   BOOL  DoOP();

};

/************************************************************/
/*                   CClientOprtGetAttr                     */
/************************************************************/
class CClientOprtGetAttr : public CClientOperation
{
private:
   
/* constructor and destructor */
public:
    CClientOprtGetAttr();
    CClientOprtGetAttr(CClientLoadMgrThread *host);
    ~CClientOprtGetAttr();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtSetAttr                     */
/************************************************************/
class CClientOprtSetAttr : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtSetAttr();
    CClientOprtSetAttr(CClientLoadMgrThread *host);
    ~CClientOprtSetAttr();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtCreatNULL                   */
/************************************************************/
class CClientOprtCreatNULL : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtCreatNULL();
    CClientOprtCreatNULL(CClientLoadMgrThread *host);
    ~CClientOprtCreatNULL();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtMkdir                     */
/************************************************************/
class CClientOprtMkdir : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtMkdir();
    CClientOprtMkdir(CClientLoadMgrThread *host);
    ~CClientOprtMkdir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtRmdir                       */
/************************************************************/
class CClientOprtRmdir : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtRmdir();
    CClientOprtRmdir(CClientLoadMgrThread *host);
    ~CClientOprtRmdir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtReadDir                     */
/************************************************************/
class CClientOprtReadDir : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtReadDir();
    CClientOprtReadDir(CClientLoadMgrThread *host);
    ~CClientOprtReadDir();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};


/************************************************************/
/*                   CClientOprtRename                     */
/************************************************************/
class CClientOprtRename : public CClientOperation
{
private:
    
/* constructor and destructor */
public:
    CClientOprtRename();
    CClientOprtRename(CClientLoadMgrThread *host);
    ~CClientOprtRename();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};

/************************************************************/
/*                   CClientOprtDelete                     */
/************************************************************/
class CClientOprtDelete : public CClientOperation
{
private:

/* constructor and destructor */
public:
    CClientOprtDelete();
    CClientOprtDelete(CClientLoadMgrThread *host);
    ~CClientOprtDelete();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};


/************************************************************/
/*                   CClientOprtGetDiskSpace                     */
/************************************************************/
class CClientOprtGetDiskSpace : public CClientOperation
{
private:
    int  pagesize;  
    int  numpages;

/* constructor and destructor */
public:
    CClientOprtGetDiskSpace();
    CClientOprtGetDiskSpace(CClientLoadMgrThread *host);
    ~CClientOprtGetDiskSpace();

/* operations */
public:
    CClientOperation *DupObj(CClientLoadMgrThread *host);
    BOOL  DoOP();
};


/************************************************************/
/*  define constant for each operation */

#define OPRT_READ           0
#define OPRT_OVERWRITE      1
#define OPRT_APPEND         2
#define OPRT_TRUNCATE       3   
#define OPRT_GETATTR        4
#define OPRT_SETATTR        5
#define OPRT_CREATENULL     6
#define OPRT_MKDIR          7
#define OPRT_RMDIR          8
#define OPRT_READDIR        9
#define OPRT_RENAME         10
#define OPRT_DELETE         11
#define OPRT_GETDISKSPACE   12
#define TOTAL               13      /* slot for totals */

#define MAX_OPNUM           TOTAL

struct Op_Map
{
    char    op_name[MAX_NAMELEN];   /* operation name */
    int     op_pcnt;                /* percentage of call target */  
    int	    target_calls;	        /* number of calls to make */
    double	req_pcnt;	            /* cumulative request percentile */
	int		req_cnt;	            /* number of requests made */
	int		target_reqs;	        /* number of req to be made */
    CClientOperation *op;           /* an instance of each operation class */
};

extern Op_Map Ops_Pcnt[MAX_OPNUM+1];

#endif